@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2019, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -28,24 +28,28 @@
 #include "mysqlshdk/include/shellcore/console.h"
 #include "mysqlshdk/include/shellcore/interrupt_handler.h"
 #include "mysqlshdk/libs/db/mysql/session.h"
+#include "mysqlshdk/libs/mysql/replication.h"
+#include "mysqlshdk/libs/utils/debug.h"
 #include "mysqlshdk/libs/utils/strformat.h"
+#include "mysqlshdk/libs/utils/utils_net.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
 namespace mysqlsh {
 namespace dba {
 
+std::shared_ptr<mysqlsh::dba::Instance> monitor_clone_recovery(
+    mysqlsh::dba::Instance *instance,
+    const mysqlshdk::db::Connection_options &post_clone_coptions,
+    const std::string &begin_time, Recovery_progress_style progress_style,
+    int restart_timeout_sec, bool is_group_member);
+
 namespace {
 
-// Return how many iterations are needed for a given timeout in seconds using
-// the given polling interval in milliseconds
-int adjust_timeout(int timeout, int poll_interval) {
-  return (timeout * poll_interval) / 1000;
-}
-
-void throw_clone_recovery_error(const mysqlshdk::mysql::IInstance &instance) {
+void throw_clone_recovery_error(const mysqlshdk::mysql::IInstance &instance,
+                                const std::string &start_time) {
   mysqlshdk::mysql::Clone_status status;
 
-  status = mysqlshdk::mysql::check_clone_status(instance);
+  status = mysqlshdk::mysql::check_clone_status(instance, start_time);
 
   assert(status.state == mysqlshdk::mysql::k_CLONE_STATE_FAILED);
 
@@ -111,7 +115,8 @@ void throw_distributed_recovery_error(
 // - 2 wait with textual info only
 // - 3 wait with progressbar
 void do_monitor_gr_recovery_status(
-    mysqlshdk::mysql::Instance *instance,
+    mysqlsh::dba::Instance *instance,
+    const mysqlshdk::db::Connection_options &post_clone_coptions,
     mysqlshdk::gr::Group_member_recovery_status method,
     const std::string &begin_time, Recovery_progress_style progress_style,
     int startup_timeout_sec, int restart_timeout_sec) {
@@ -133,9 +138,11 @@ void do_monitor_gr_recovery_status(
           80));
       console->print_info();
       if (method == mysqlshdk::gr::Group_member_recovery_status::CLONE_ERROR) {
-        throw_clone_recovery_error(*instance);
+        throw_clone_recovery_error(*instance, begin_time);
       } else if (progress_style != Recovery_progress_style::NOWAIT) {
-        monitor_clone_recovery(instance, progress_style, restart_timeout_sec);
+        Scoped_instance new_instance(
+            monitor_clone_recovery(instance, post_clone_coptions, begin_time,
+                                   progress_style, restart_timeout_sec, true));
 
         // After a clone, a distributed recovery is done by GR, but there's a
         // time gap between clone finishing and the next recovery starting.
@@ -143,7 +150,8 @@ void do_monitor_gr_recovery_status(
         // switches away from RECOVERING, so we need to poll again until we know
         // what's happening.
         monitor_post_clone_gr_recovery_status(
-            instance, begin_time, progress_style, startup_timeout_sec);
+            new_instance.get(), post_clone_coptions, begin_time, progress_style,
+            startup_timeout_sec);
       } else {
         console->print_info(
             "State recovery will continue in background, you may monitor its "
@@ -156,7 +164,7 @@ void do_monitor_gr_recovery_status(
       // if clone recovery fails, GR can fallback to distributed recovery
       try {
         mysqlshdk::mysql::Clone_status status =
-            mysqlshdk::mysql::check_clone_status(*instance);
+            mysqlshdk::mysql::check_clone_status(*instance, begin_time);
 
         if (status.state == mysqlshdk::mysql::k_CLONE_STATE_FAILED) {
           console->print_warning(
@@ -164,13 +172,13 @@ void do_monitor_gr_recovery_status(
               status.error + " (" + std::to_string(status.error_n) + ")");
           console->print_info("A fallback method will be used.");
         }
-      } catch (const mysqlshdk::db::Error &e) {
+      } catch (const shcore::Error &e) {
         if (e.code() != ER_BAD_TABLE_ERROR) {
           log_debug("Error querying clone status: %s", e.format().c_str());
         }
       }
       console->print_info(
-          bold("Incremental distributed state recovery is now in progress."));
+          bold("Incremental state recovery is now in progress."));
       console->print_info();
       if (method ==
           mysqlshdk::gr::Group_member_recovery_status::DISTRIBUTED_ERROR) {
@@ -229,8 +237,7 @@ mysqlshdk::gr::Group_member_recovery_status wait_recovery_start(
   timeout_sec = adjust_timeout(timeout_sec, k_recovery_status_poll_interval_ms);
   bool reconnect = true;
 
-  mysqlshdk::mysql::Instance instance;
-  std::shared_ptr<mysqlshdk::db::ISession> session;
+  Scoped_instance instance;
 
   bool stop = false;
   shcore::Interrupt_handler intr([&stop]() {
@@ -240,8 +247,8 @@ mysqlshdk::gr::Group_member_recovery_status wait_recovery_start(
   while (timeout_sec > 0 && !stop) {
     if (reconnect) {
       try {
-        wait_server_startup(instance_def, &instance, timeout_sec,
-                            Recovery_progress_style::NOWAIT);
+        instance = Scoped_instance(wait_server_startup(
+            instance_def, timeout_sec, Recovery_progress_style::NOWAIT));
         reconnect = false;
       } catch (const shcore::Exception &e) {
         if (e.code() == SHERR_DBA_SERVER_RESTART_TIMEOUT) break;
@@ -252,13 +259,13 @@ mysqlshdk::gr::Group_member_recovery_status wait_recovery_start(
     if (!reconnect) {
       try {
         mysqlshdk::gr::Group_member_recovery_status rm =
-            mysqlshdk::gr::detect_recovery_status(instance, begin_time);
+            mysqlshdk::gr::detect_recovery_status(*instance, begin_time);
         if (rm != mysqlshdk::gr::Group_member_recovery_status::UNKNOWN) {
           // We keep trying until we can detect which method is in use
           return rm;
         }
         reconnect = false;
-      } catch (const mysqlshdk::db::Error &err) {
+      } catch (const shcore::Error &err) {
         log_warning("During recovery start check: %s", err.what());
         if (mysqlshdk::db::is_mysql_client_error(err.code()) ||
             err.code() == ER_SERVER_SHUTDOWN) {
@@ -274,13 +281,75 @@ mysqlshdk::gr::Group_member_recovery_status wait_recovery_start(
     timeout_sec--;
   }
 
-  if (session) session->close();
-
   if (stop) throw stop_monitoring();
 
   return mysqlshdk::gr::Group_member_recovery_status::UNKNOWN;
 }
 
+std::shared_ptr<mysqlsh::dba::Instance> wait_clone_start(
+    const mysqlshdk::db::Connection_options &instance_def,
+    const std::string &begin_time, int timeout_sec) {
+  // We wait in this loop until something shows up in PFS.clone_status
+  timeout_sec = adjust_timeout(timeout_sec, k_recovery_status_poll_interval_ms);
+
+  std::shared_ptr<mysqlsh::dba::Instance> out_instance;
+
+  bool reconnect = true;
+
+  bool stop = false;
+  shcore::Interrupt_handler intr([&stop]() {
+    stop = true;
+    return true;
+  });
+
+  auto poll_interval_ms = k_recovery_status_poll_interval_ms;
+  DBUG_EXECUTE_IF("clone_rig_poll_interval", { poll_interval_ms = 10; });
+
+  while (timeout_sec > 0 && !stop) {
+    if (reconnect) {
+      try {
+        out_instance = wait_server_startup(instance_def, timeout_sec,
+                                           Recovery_progress_style::NOWAIT);
+        reconnect = false;
+      } catch (const shcore::Exception &e) {
+        if (e.code() == SHERR_DBA_SERVER_RESTART_TIMEOUT) break;
+        throw;
+      }
+    }
+
+    if (!reconnect) {
+      try {
+        mysqlshdk::mysql::Clone_status status =
+            mysqlshdk::mysql::check_clone_status(*out_instance, begin_time);
+
+        if (!status.state.empty() && !status.stages.empty()) {
+          // We keep trying until we can detect clone has started
+          break;
+        }
+        reconnect = false;
+      } catch (const shcore::Error &err) {
+        log_warning("Error during clone start check: %s", err.format().c_str());
+
+        if (mysqlshdk::db::is_mysql_client_error(err.code()) ||
+            err.code() == ER_SERVER_SHUTDOWN) {
+          // client errors are probably a lost connection, which may mean the
+          // instance is restarting
+          reconnect = true;
+        } else {
+          throw;
+        }
+      }
+    }
+
+    shcore::sleep_ms(poll_interval_ms);
+    timeout_sec--;
+  }
+
+  if (stop) throw stop_monitoring();
+
+  return out_instance;
+}
+
 void monitor_distributed_recovery(const mysqlshdk::mysql::IInstance &instance,
                                   Recovery_progress_style /*progress_style*/) {
   // TODO(.) - show progress
@@ -321,9 +390,10 @@ void monitor_distributed_recovery(const mysqlshdk::mysql::IInstance &instance,
           show_distributed_recovery_error(instance, last_error_time, &channel);
 
       if (first) {
-        console->print_note("'" + instance.descr() +
-                            "' is being recovered from '" + channel.host + ":" +
-                            std::to_string(channel.port) + "'");
+        console->print_note(
+            "'" + instance.descr() + "' is being recovered from '" +
+            mysqlshdk::utils::make_host_and_port(channel.host, channel.port) +
+            "'");
         first = false;
       }
     }
@@ -338,11 +408,35 @@ void monitor_distributed_recovery(const mysqlshdk::mysql::IInstance &instance,
   console->print_info();
 }
 
-void monitor_clone_recovery(mysqlshdk::mysql::Instance *instance,
-                            Recovery_progress_style progress_style,
-                            int restart_timeout_sec) {
+void monitor_standalone_clone_instance(
+    const mysqlshdk::db::Connection_options &instance_def,
+    const mysqlshdk::db::Connection_options &post_clone_coptions,
+    const std::string &begin_time, Recovery_progress_style progress_style,
+    int startup_timeout_sec, int restart_timeout_sec) {
+  auto console = current_console();
+
+  console->print_info(
+      "Waiting for clone process of the new member to complete. "
+      "Press ^C to abort the operation.");
+
+  log_info("Waiting for clone process to start at %s...",
+           instance_def.uri_endpoint().c_str());
+
+  Scoped_instance instance(
+      wait_clone_start(instance_def, begin_time, startup_timeout_sec));
+
+  monitor_clone_recovery(instance.get(), post_clone_coptions, begin_time,
+                         progress_style, restart_timeout_sec, false);
+}
+
+std::shared_ptr<mysqlsh::dba::Instance> monitor_clone_recovery(
+    mysqlsh::dba::Instance *instance,
+    const mysqlshdk::db::Connection_options &post_clone_coptions,
+    const std::string &begin_time, Recovery_progress_style progress_style,
+    int restart_timeout_sec, bool is_group_member) {
   auto console = current_console();
   bool wait_restart = false;
+  bool ignore_cancel = false;
   Clone_progress progress(progress_style);
 
   bool stop = false;
@@ -351,20 +445,27 @@ void monitor_clone_recovery(mysqlshdk::mysql::Instance *instance,
     return true;
   });
 
+  auto poll_interval_ms = k_clone_status_poll_interval_ms;
+  DBUG_EXECUTE_IF("clone_rig_poll_interval", { poll_interval_ms = 10; });
+
   bool first = true;
   console->print_info("* Waiting for clone to finish...");
   while (!stop) {
     mysqlshdk::mysql::Clone_status status;
 
     try {
-      status = mysqlshdk::mysql::check_clone_status(*instance);
+      status = mysqlshdk::mysql::check_clone_status(*instance, begin_time);
       if (first) {
         console->print_note(instance->descr() + " is being cloned from " +
                             status.source);
         first = false;
       }
-    } catch (const mysqlshdk::db::Error &e) {
+    } catch (const shcore::Error &e) {
       if (e.code() == ER_SERVER_SHUTDOWN) {
+        // From the very same moment the instance starts the shutdown, aborting
+        // the process with ctrl-c must be disabled
+        ignore_cancel = true;
+
         progress.on_restart();
 
         console->print_info();
@@ -397,21 +498,48 @@ void monitor_clone_recovery(mysqlshdk::mysql::Instance *instance,
       break;
     }
 
-    shcore::sleep_ms(k_clone_status_poll_interval_ms);
+    shcore::sleep_ms(poll_interval_ms);
   }
-  if (stop) throw stop_monitoring();
+  if (stop && !ignore_cancel) throw stop_monitoring();
+
+  std::shared_ptr<mysqlsh::dba::Instance> new_instance;
 
   if (wait_restart) {
+    ignore_cancel = true;
+
     // Wait for the server to start again
-    wait_server_startup(instance->get_connection_options(), instance,
-                        restart_timeout_sec, progress_style);
+    try {
+      new_instance = wait_server_startup(post_clone_coptions,
+                                         restart_timeout_sec, progress_style);
+    } catch (const shcore::Exception &e) {
+      if (e.code() == SHERR_DBA_SERVER_RESTART_TIMEOUT) {
+        throw restart_timeout();
+      } else {
+        throw;
+      }
+    }
+
     console->print_info("* " + instance->get_canonical_address() +
                         " has restarted, waiting for clone to finish...");
+
+    // if start_on_boot is disabled, start GR by hand
+    if (is_group_member &&
+        !new_instance->get_sysvar_bool("group_replication_start_on_boot")
+             .get_safe()) {
+      new_instance->execute("START GROUP_REPLICATION");
+    }
   }
   // Wait for clone recovery to finish
   while (!stop) {
     mysqlshdk::mysql::Clone_status status;
-    status = mysqlshdk::mysql::check_clone_status(*instance);
+
+    // If a restart happened, use the new pointer. Otherwise, we must use the
+    // old pointer to not cause a segfault
+    if (new_instance) {
+      status = mysqlshdk::mysql::check_clone_status(*new_instance, begin_time);
+    } else {
+      status = mysqlshdk::mysql::check_clone_status(*instance, begin_time);
+    }
 
     try {
       progress.update(status);
@@ -452,12 +580,18 @@ void monitor_clone_recovery(mysqlshdk::mysql::Instance *instance,
     }
     shcore::sleep_ms(k_clone_status_poll_interval_ms);
   }
-  if (stop) throw stop_monitoring();
+  if (stop && !ignore_cancel) throw stop_monitoring();
+
+  // TODO(miguel/alfredo): refactor this monitoring code to pass the less
+  // possible pointers around
+  return new_instance;
 }
 
 void monitor_post_clone_gr_recovery_status(
-    mysqlshdk::mysql::Instance *instance, const std::string &begin_time,
-    Recovery_progress_style progress_style, int startup_timeout_sec) {
+    mysqlsh::dba::Instance *instance,
+    const mysqlshdk::db::Connection_options &post_clone_coptions,
+    const std::string &begin_time, Recovery_progress_style progress_style,
+    int startup_timeout_sec) {
   bool stop = false;
   shcore::Interrupt_handler intr([&stop]() {
     stop = true;
@@ -469,22 +603,24 @@ void monitor_post_clone_gr_recovery_status(
   mysqlshdk::gr::Group_member_recovery_status rm =
       mysqlshdk::gr::Group_member_recovery_status::UNKNOWN;
 
-  startup_timeout_sec =
+  int startup_timeout_iters =
       adjust_timeout(startup_timeout_sec, k_recovery_status_poll_interval_ms);
-  while (startup_timeout_sec > 0 && !stop) {
+
+  while (startup_timeout_iters > 0 && !stop) {
     try {
       rm = mysqlshdk::gr::detect_recovery_status(*instance, begin_time);
       if (rm != mysqlshdk::gr::Group_member_recovery_status::CLONE) {
-        do_monitor_gr_recovery_status(instance, rm, begin_time, progress_style,
+        do_monitor_gr_recovery_status(instance, post_clone_coptions, rm,
+                                      begin_time, progress_style,
                                       startup_timeout_sec, 0);
         break;
       }
-    } catch (const mysqlshdk::db::Error &err) {
+    } catch (const shcore::Error &err) {
       log_warning("During post-clone recovery start check: %s", err.what());
       throw;
     }
     shcore::sleep_ms(k_recovery_status_poll_interval_ms);
-    startup_timeout_sec--;
+    startup_timeout_iters--;
   }
 
   if (stop) throw stop_monitoring();
@@ -492,6 +628,7 @@ void monitor_post_clone_gr_recovery_status(
 
 void monitor_gr_recovery_status(
     const mysqlshdk::db::Connection_options &instance_def,
+    const mysqlshdk::db::Connection_options &post_clone_coptions,
     const std::string &begin_time, Recovery_progress_style progress_style,
     int startup_timeout_sec, int restart_timeout_sec) {
   auto console = current_console();
@@ -507,28 +644,43 @@ void monitor_gr_recovery_status(
         "stop monitoring and let it continue in background.");
   }
 
+  // first time, try with target credentials
+  auto connection_options = instance_def;
+
   try {
     log_info("Waiting for GR recovery to start for %s...",
              instance_def.uri_endpoint().c_str());
 
-    mysqlshdk::gr::Group_member_recovery_status method =
-        wait_recovery_start(instance_def, begin_time, startup_timeout_sec);
+    for (int attempt = 0; attempt < 2; ++attempt) {
+      try {
+        mysqlshdk::gr::Group_member_recovery_status method =
+            wait_recovery_start(connection_options, begin_time,
+                                startup_timeout_sec);
 
-    auto session = mysqlshdk::db::mysql::Session::create();
-    session->connect(instance_def);
-    mysqlshdk::mysql::Instance instance(session);
+        auto instance = mysqlsh::dba::Instance::connect(instance_def);
 
-    do_monitor_gr_recovery_status(&instance, method, begin_time, progress_style,
-                                  startup_timeout_sec, restart_timeout_sec);
+        do_monitor_gr_recovery_status(instance.get(), post_clone_coptions,
+                                      method, begin_time, progress_style,
+                                      startup_timeout_sec, restart_timeout_sec);
+        break;
+      } catch (const shcore::Error &e) {
+        if (e.code() == ER_ACCESS_DENIED_ERROR && attempt == 0 &&
+            post_clone_coptions.get_user() != instance_def.get_user()) {
+          // try again with cluster-wide credentials
+          connection_options = post_clone_coptions;
+          continue;
+        }
+        throw;
+      }
+    }
   } catch (const stop_monitoring &) {
     console->print_info();
     console->print_note(
         "Monitoring of the recovery process was canceled. The recovery itself "
         "will continue executing in background. Use <Cluster>.status() to "
         "check progress.");
-  } catch (const mysqlshdk::db::Error &e) {
-    throw shcore::Exception::mysql_error_with_code_and_state(e.what(), e.code(),
-                                                             e.sqlstate());
+  } catch (const shcore::Error &e) {
+    throw shcore::Exception::mysql_error_with_code(e.what(), e.code());
   }
 }
 
