@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -24,11 +24,14 @@
 #ifndef MODULES_MOD_EXTENSIBLE_OBJECT_H_
 #define MODULES_MOD_EXTENSIBLE_OBJECT_H_
 
+#include <map>
 #include <memory>
 #include <string>
 #include <vector>
 
-#include "scripting/types_cpp.h"
+#include "mysqlshdk/include/scripting/types.h"
+#include "mysqlshdk/include/scripting/types_cpp.h"
+#include "mysqlshdk/include/shellcore/utils_help.h"
 
 namespace mysqlsh {
 /**
@@ -111,11 +114,16 @@ struct Parameter_definition {
  */
 struct Function_definition : public Member_definition {
   using Parameters = std::vector<std::shared_ptr<Parameter_definition>>;
+  using Examples = std::vector<shcore::Help_registry::Example>;
   Function_definition() {}
   Function_definition(const std::string &n, const Parameters &p,
-                      const std::string &b, const std::vector<std::string> &d)
-      : Member_definition(n, b, d), parameters(p) {}
+                      const std::string &b, const std::vector<std::string> &d,
+                      const Examples &e = {})
+      : Member_definition(n, b, d), parameters(p), examples(e) {}
   Parameters parameters;
+
+  Examples examples;
+  bool cli_enabled = false;
 };
 
 /**
@@ -147,7 +155,8 @@ class Extensible_object
   shcore::Value call(const std::string &name,
                      const shcore::Argument_list &args) override;
   shcore::Value call_advanced(const std::string &name,
-                              const shcore::Argument_list &args) override;
+                              const shcore::Argument_list &args,
+                              const shcore::Dictionary_t &kwargs = {}) override;
 
   /**
    * Registers a new member on this object.
@@ -328,6 +337,7 @@ class Extensible_object
    * @param brief A brief description of the function.
    * @param params A list defining the parameters for the function.
    * @param details A list defining the help details for the function.
+   * @param examples A list defining the examples for the function.
    *
    * The entries in the parameter list should come in the next format:
    *
@@ -339,9 +349,11 @@ class Extensible_object
    * Each entry help entry in params and details is registered and follows
    * the same rules as the REGISTER_HELP macro.
    */
-  void register_function_help(const std::string &name, const std::string &brief,
-                              const std::vector<std::string> &params,
-                              const std::vector<std::string> &details);
+  void register_function_help(
+      const std::string &name, const std::string &brief,
+      const std::vector<std::string> &params,
+      const std::vector<std::string> &details,
+      const Function_definition::Examples &examples = {});
 
   /**
    * Sets the registered status of this object to true.
@@ -364,6 +376,12 @@ class Extensible_object
   }
 
   bool is_registered() const { return m_registered; }
+  void enable_cli() {
+    m_cli_enabled = true;
+    auto parent = get_parent();
+    if (parent) parent->enable_cli();
+  }
+  bool cli_enabled() const { return m_cli_enabled; }
 
   std::string get_name() const { return m_name; }
   std::string get_qualified_name() const { return m_qualified_name; }
@@ -383,10 +401,6 @@ class Extensible_object
       shcore::Option_unpacker *unpacker) const;
 
  private:
-  static std::set<shcore::Value_type> s_allowed_member_types;
-  static std::set<std::string> s_allowed_param_types;
-  static std::map<std::string, shcore::Value_type> s_type_mapping;
-
   static void validate_name(const std::string &name, const std::string &label,
                             bool custom_names);
 
@@ -397,7 +411,7 @@ class Extensible_object
       const std::shared_ptr<Function_definition> &parameters);
 
   shcore::Value_type map_type(const std::string &type,
-                              const std::set<std::string> &allowed_types);
+                              const std::set<std::string> &allowed_types) const;
   std::shared_ptr<Parameter_definition> parse_parameter(
       const shcore::Dictionary_t &definition,
       shcore::Parameter_context *context,
@@ -427,6 +441,7 @@ class Extensible_object
   std::string m_name;
   std::string m_qualified_name;
   bool m_registered;
+  bool m_cli_enabled = false;
   size_t m_detail_sequence;
   std::map<std::string, std::shared_ptr<Extensible_object>> m_children;
   shcore::Value::Map_type m_members;
@@ -439,32 +454,4 @@ class Extensible_object
 };
 }  // namespace mysqlsh
 
-namespace shcore {
-
-template <>
-struct Type_info<std::shared_ptr<mysqlsh::Extensible_object>> {
-  static std::shared_ptr<mysqlsh::Extensible_object> to_native(
-      const shcore::Value &in) {
-    std::shared_ptr<mysqlsh::Extensible_object> object;
-
-    if (in.type == shcore::Object)
-      object = in.as_object<mysqlsh::Extensible_object>();
-
-    if (!object) {
-      throw shcore::Exception::type_error(
-          "Invalid typecast: extension object expected.");
-    }
-
-    return object;
-  }
-  static Value_type vtype() { return shcore::Object; }
-  static const char *code() { return "O"; }
-  static std::shared_ptr<mysqlsh::Extensible_object> default_value() {
-    return std::shared_ptr<mysqlsh::Extensible_object>();
-  }
-  static std::string desc() { return "an extension object"; }
-};
-
-}  // namespace shcore
-
 #endif  // MODULES_MOD_EXTENSIBLE_OBJECT_H_
