@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -23,10 +23,12 @@
 
 #include "modules/mod_utils.h"
 
+#include <map>
 #include <string>
 #include <utility>
 
 #include "mysqlshdk/include/scripting/obj_date.h"
+#include "mysqlshdk/include/scripting/type_info/custom.h"
 #include "mysqlshdk/include/scripting/types.h"
 #include "mysqlshdk/include/shellcore/base_shell.h"
 #include "mysqlshdk/include/shellcore/console.h"
@@ -34,12 +36,16 @@
 #include "mysqlshdk/include/shellcore/shell_options.h"
 #include "mysqlshdk/libs/db/mysql/session.h"
 #include "mysqlshdk/libs/db/mysqlx/session.h"
+#include "mysqlshdk/libs/utils/fault_injection.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
+#include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 #include "mysqlshdk/shellcore/credential_manager.h"
 
 namespace mysqlsh {
 
+namespace {
+
 /**
  * Retrieves the connection data from a String
  *
@@ -47,11 +53,15 @@ namespace mysqlsh {
  *
  * @return a Connection_options object with the connection data
  */
-mysqlshdk::db::Connection_options get_connection_options(
-    const std::string &instance_def) {
+Connection_options get_connection_options(const std::string &instance_def) {
   if (instance_def.empty()) throw std::invalid_argument("Invalid URI: empty.");
 
-  return shcore::get_connection_options(instance_def, false);
+  auto ret_val = shcore::get_connection_options(instance_def, false);
+  for (const auto &warning : ret_val.get_warnings())
+    mysqlsh::current_console()->print_warning(warning);
+  ret_val.clear_warnings();
+
+  return ret_val;
 }
 
 /**
@@ -61,10 +71,8 @@ mysqlshdk::db::Connection_options get_connection_options(
  *
  * @return a Connection_options object with the connection data
  */
-mysqlshdk::db::Connection_options get_connection_options(
+Connection_options get_connection_options(
     const shcore::Dictionary_t &instance_def) {
-  mysqlshdk::db::Connection_options ret_val;
-
   if (instance_def == nullptr || instance_def->size() == 0) {
     throw std::invalid_argument(
         "Invalid connection options, no options provided.");
@@ -73,16 +81,48 @@ mysqlshdk::db::Connection_options get_connection_options(
   shcore::Argument_map connection_map(*instance_def);
 
   std::set<std::string> mandatory;
-  if (!connection_map.has_key(mysqlshdk::db::kSocket)) {
-    mandatory.insert(mysqlshdk::db::kHost);
-  }
+  // host defaults to localhost, so it's not mandatory
+  // if (!connection_map.has_key(mysqlshdk::db::kSocket)) {
+  //   mandatory.insert(mysqlshdk::db::kHost);
+  // }
+
+  Connection_options ret_val;
 
   connection_map.ensure_keys(
-      mandatory, mysqlshdk::db::connection_attributes, "connection options",
+      mandatory, mysqlshdk::db::connection_attributes(), "connection options",
       ret_val.get_mode() == mysqlshdk::db::Comparison_mode::CASE_SENSITIVE);
 
+  // First we load uri if possible, so we can override it later with explicit
+  // options.
   for (auto &option : *instance_def) {
-    if (ret_val.compare(option.first, mysqlshdk::db::kHost) == 0) {
+    if (ret_val.compare(option.first, mysqlshdk::db::kUri) == 0) {
+      ret_val = Connection_options(connection_map.string_at(option.first));
+      break;
+    }
+  }
+
+  bool (*fn_pt)(const std::string &a, const std::string &b);
+  if (ret_val.get_mode() == mysqlshdk::db::Comparison_mode::CASE_SENSITIVE) {
+    fn_pt = [](const std::string &a, const std::string &b) {
+      return a.compare(b) < 0;
+    };
+  } else {
+    fn_pt = [](const std::string &a, const std::string &b) {
+      return shcore::str_casecmp(a, b) < 0;
+    };
+  }
+
+  auto customset = std::set<std::string, decltype(fn_pt)>(fn_pt);
+
+  customset.insert(mysqlshdk::db::ssh_uri_connection_attributes.begin(),
+                   mysqlshdk::db::ssh_uri_connection_attributes.end());
+
+  for (auto &option : *instance_def) {
+    if (customset.find(option.first) != customset.end()) {
+      continue;  // those will be handled in different place
+    } else if (ret_val.compare(option.first, mysqlshdk::db::kUri) == 0) {
+      continue;  // those were already handled
+    } else if (ret_val.compare(option.first, mysqlshdk::db::kHost) == 0) {
       const auto &host = connection_map.string_at(option.first);
 
       if (host.empty()) {
@@ -99,6 +139,9 @@ mysqlshdk::db::Connection_options get_connection_options(
 #else   // !_WIN32
       ret_val.set_socket(sock);
 #endif  // !_WIN32
+    } else if (ret_val.compare(option.first,
+                               mysqlshdk::db::kCompressionLevel) == 0) {
+      ret_val.set_compression_level(connection_map.int_at(option.first));
     } else if (ret_val.compare(option.first, mysqlshdk::db::kConnectTimeout) ==
                0) {
       // Additional connection options are internally stored as strings.
@@ -135,71 +178,79 @@ mysqlshdk::db::Connection_options get_connection_options(
       ret_val.set(option.first, connection_map.string_at(option.first));
     }
   }
+
+  // The ssh has to be set as the last step cause we need the host and port to
+  // be set, otherwise we can't use the tunneling.
+  auto ssh_data = get_ssh_options(instance_def);
+  // don't use has_data here as there's no remote_host yet in the
+  // ssh_connection_options, those will be set inside of the set_ssh_options.
+  if (ssh_data.has_host()) ret_val.set_ssh_options(ssh_data);
+
+  for (const auto &warning : ret_val.get_warnings())
+    mysqlsh::current_console()->print_warning(warning);
+  ret_val.clear_warnings();
+
   return ret_val;
 }
 
-/**
- * This function will retrieve the connection data from the received arguments
- * Connection data can be specified in one of:
- * - Dictionary
- * - URI
- *
- * A common pattern is functions with a signature as follows:
- *
- * function name(connection[, password])
- * function name(connection[, options])
- *
- * On both functions connection may be either a dictionary or a string (URI)
- * An optional password parameter may be defined, this will override the
- * password defined in connection.
- *
- * An optional options parameter may be defined, it may contain a password
- * entry that wold override the one defined in connection.
- *
- * Conflicting options will also be validated.
- */
-mysqlshdk::db::Connection_options get_connection_options(
-    const shcore::Argument_list &args, PasswordFormat format) {
-  mysqlshdk::db::Connection_options ret_val;
+}  // namespace
 
-  try {
-    if (args.size() > 0 && args[0].type == shcore::String) {
-      std::string uri = args.string_at(0);
+Connection_options get_connection_options(const shcore::Value &v) {
+  if (shcore::String == v.type) {
+    return get_connection_options(v.get_string());
+  } else if (shcore::Map == v.type) {
+    return get_connection_options(v.as_map());
+  } else {
+    throw shcore::Exception::type_error(
+        "Invalid connection options, expected either a URI or a Connection "
+        "Options Dictionary");
+  }
+}
 
-      ret_val = get_connection_options(uri);
-    } else if (args.size() > 0 && args[0].type == shcore::Map) {
-      shcore::Value::Map_type_ref options = args.map_at(0);
-
-      ret_val = get_connection_options(options);
-    } else {
-      throw std::invalid_argument(
-          "Invalid connection options, expected either "
-          "a URI or a Dictionary.");
-    }
-
-    // Some functions override the password in a second parameter
-    // which could be either a string or an options map, such behavior is
-    // handled here
-    if (format != PasswordFormat::NONE && args.size() > 1) {
-      if (format == PasswordFormat::OPTIONS) {
-        set_password_from_map(&ret_val, args.map_at(1));
-      } else if (format == PasswordFormat::STRING) {
-        ret_val.clear_password();
-        ret_val.set_password(args.string_at(1));
-      }
-    }
-  } catch (const std::invalid_argument &error) {
-    throw shcore::Exception::argument_error(error.what());
+mysqlshdk::ssh::Ssh_connection_options get_ssh_options(
+    const shcore::Dictionary_t &instance_def) {
+  if (instance_def == nullptr || instance_def->size() == 0) {
+    throw std::invalid_argument("Invalid SSH options, no options provided.");
   }
 
-  return ret_val;
+  shcore::Argument_map connection_map(*instance_def);
+  mysqlshdk::ssh::Ssh_connection_options ssh_config;
+
+  for (auto &option : *instance_def) {
+    if (ssh_config.compare(option.first, mysqlshdk::db::kSsh) == 0) {
+      ssh_config = mysqlshdk::ssh::Ssh_connection_options(
+          connection_map.string_at(option.first));
+    } else if (ssh_config.compare(option.first,
+                                  mysqlshdk::db::kSshConfigFile) == 0) {
+      ssh_config.clear_config_file();
+      ssh_config.set_config_file(connection_map.string_at(option.first));
+    } else if (ssh_config.compare(option.first,
+                                  mysqlshdk::db::kSshIdentityFile) == 0) {
+      ssh_config.clear_key_file();
+      ssh_config.set_key_file(connection_map.string_at(option.first));
+    } else if (ssh_config.compare(option.first,
+                                  mysqlshdk::db::kSshIdentityFilePassword) ==
+               0) {
+      ssh_config.set_key_file_password(connection_map.string_at(option.first));
+    } else if (ssh_config.compare(option.first, mysqlshdk::db::kSshPassword) ==
+               0) {
+      ssh_config.set_password(connection_map.string_at(option.first));
+    }
+  }
+
+  return ssh_config;
 }
 
-void SHCORE_PUBLIC set_password_from_map(
-    Connection_options *options, const shcore::Value::Map_type_ref &map) {
+void SHCORE_PUBLIC set_password_from_map(Connection_options *options,
+                                         const shcore::Dictionary_t &map) {
+  if (!options || !map) {
+    return;
+  }
+
   bool override_pwd = false;
   std::string key;
-  for (auto option : *map) {
+
+  for (const auto &option : *map) {
     if (!options->compare(option.first, mysqlshdk::db::kPassword) ||
         !options->compare(option.first, mysqlshdk::db::kDbPassword)) {
       // Will allow one override, a second means the password option was
@@ -217,6 +268,14 @@ void SHCORE_PUBLIC set_password_from_map(
   if (override_pwd) map->erase(key);
 }
 
+void set_password_from_string(Connection_options *options,
+                              const char *password) {
+  if (options && password) {
+    options->clear_password();
+    options->set_password(password);
+  }
+}
+
 void SHCORE_PUBLIC set_user_from_map(Connection_options *options,
                                      const shcore::Value::Map_type_ref &map) {
   bool override_user = false;
@@ -303,6 +362,10 @@ shcore::Value::Map_type_ref get_connection_map(
     if (ssl.has_tls_version())
       (*map)[mysqlshdk::db::kSslTlsVersion] =
           shcore::Value(ssl.get_tls_version());
+
+    if (ssl.has_tls_ciphersuites())
+      (*map)[mysqlshdk::db::kSslTlsCiphersuites] =
+          shcore::Value(ssl.get_tls_ciphersuites());
   }
 
   for (auto &option : connection_options.get_extra_options()) {
@@ -373,16 +436,20 @@ std::shared_ptr<mysqlshdk::db::ISession> create_and_connect(
           code == CR_CONNECTION_ERROR ||  // No connection could be made because
                                           // the target machine actively refused
                                           // it connecting to host:port
-          code == CR_SERVER_GONE_ERROR) {  // MySQL server has gone away
+          code == CR_SERVER_GONE_ERROR ||  // MySQL server has gone away
                                            // (randomly sent by libmysqlx)
+          (CR_X_UNSUPPORTED_OPTION_VALUE == code &&       // auth-method was not
+           strstr(e.what(), "authentication method"))) {  // valid for X proto
         type = mysqlsh::SessionType::Classic;
         copy.clear_scheme();
         copy.set_scheme("mysql");
 
         // Since this is an unexpected error, we store the message to be
         // logged in case the classic session connection fails too
-        if (code == CR_SERVER_GONE_ERROR)
+        if (code == CR_SERVER_GONE_ERROR ||
+            CR_X_UNSUPPORTED_OPTION_VALUE == code) {
           connection_error.append("X protocol error: ").append(e.what());
+        }
       } else {
         throw;
       }
@@ -404,7 +471,7 @@ std::shared_ptr<mysqlshdk::db::ISession> create_and_connect(
 
   try {
     session->connect(copy);
-  } catch (const shcore::Exception &e) {
+  } catch (const mysqlshdk::db::Error &e) {
     if (connection_error.empty()) {
       throw;
     } else {
@@ -428,6 +495,9 @@ std::shared_ptr<mysqlshdk::db::ISession> create_session(
     return true;
   });
 
+  log_info("About to connect to MySQL at: %s",
+           connection_options.as_uri().c_str());
+
   auto session = create_and_connect(connection_options);
 
   if (cancelled) throw shcore::cancelled("Cancelled");
@@ -454,64 +524,77 @@ void password_prompt(Connection_options *options) {
 std::shared_ptr<mysqlshdk::db::ISession> establish_session(
     const Connection_options &options, bool prompt_for_password,
     bool prompt_in_loop) {
-  Connection_options copy = options;
+  FI_SUPPRESS(mysql);
+  FI_SUPPRESS(mysqlx);
 
-  copy.set_default_connection_data();
+  try {
+    Connection_options copy = options;
 
-  if (!copy.has_password()) {
-    if (shcore::Credential_manager::get().get_password(&copy)) {
-      try {
-        return create_session(copy);
-      } catch (const mysqlshdk::db::Error &e) {
-        if (e.code() != ER_ACCESS_DENIED_ERROR) {
-          throw;
-        } else {
-          copy.clear_password();
-          shcore::Credential_manager::get().remove_password(copy);
-          log_info(
-              "Connection to \"%s\" could not be established using the "
-              "stored "
-              "password: %s. "
-              "Invalid password has been erased.",
-              copy.as_uri(mysqlshdk::db::uri::formats::user_transport())
-                  .c_str(),
-              e.format().c_str());
+    copy.set_default_data();
+
+    auto &ssh = copy.get_ssh_options_handle();
+    if (ssh.has_data()) {
+      mysqlshdk::ssh::current_ssh_manager()->create_tunnel(&ssh);
+    }
+
+    if (!copy.has_password() && copy.has_user()) {
+      if (shcore::Credential_manager::get().get_password(&copy)) {
+        try {
+          return create_session(copy);
+        } catch (const mysqlshdk::db::Error &e) {
+          if (e.code() != ER_ACCESS_DENIED_ERROR) {
+            throw;
+          } else {
+            copy.clear_password();
+            shcore::Credential_manager::get().remove_password(copy);
+            log_info(
+                "Connection to \"%s\" could not be established using the "
+                "stored password: %s. Invalid password has been erased.",
+                copy.as_uri(mysqlshdk::db::uri::formats::user_transport())
+                    .c_str(),
+                e.format().c_str());
+          }
         }
       }
     }
-  }
 
-  if (prompt_for_password) {
-    do {
-      bool prompted_for_password = false;
+    if (prompt_for_password) {
+      do {
+        bool prompted_for_password = false;
 
-      if (!copy.has_password()) {
-        password_prompt(&copy);
-        prompted_for_password = true;
-      }
-
-      try {
-        auto session = create_session(copy);
-
-        if (prompted_for_password) {
-          // save password using the same connection options as the ones used
-          // to fetch the password from the secret storage
-          shcore::Credential_manager::get().save_password(copy);
+        if (!copy.has_password() && copy.has_user() &&
+            !copy.is_auth_method(mysqlshdk::db::kAuthMethodKerberos) &&
+            !copy.is_auth_method(mysqlshdk::db::kAuthMethodOci)) {
+          password_prompt(&copy);
+          prompted_for_password = true;
         }
 
-        return session;
-      } catch (const mysqlshdk::db::Error &e) {
-        if (!prompt_in_loop || e.code() != ER_ACCESS_DENIED_ERROR) {
-          throw;
-        } else {
-          copy.clear_password();
-          current_console()->print_error(e.format());
-        }
-      }
-    } while (prompt_in_loop);
-  }
+        try {
+          auto session = create_session(copy);
 
-  return create_session(copy);
+          if (prompted_for_password) {
+            // save password using the same connection options as the ones used
+            // to fetch the password from the secret storage
+            shcore::Credential_manager::get().save_password(copy);
+          }
+
+          return session;
+        } catch (const mysqlshdk::db::Error &e) {
+          if (!prompt_in_loop || e.code() != ER_ACCESS_DENIED_ERROR) {
+            throw;
+          } else {
+            copy.clear_password();
+            current_console()->print_error(e.format());
+          }
+        }
+      } while (prompt_in_loop);
+    }
+
+    return create_session(copy);
+  } catch (const mysqlshdk::db::Error &e) {
+    throw shcore::Exception::mysql_error_with_code_and_state(e.what(), e.code(),
+                                                             e.sqlstate());
+  }
 }
 
 std::shared_ptr<mysqlshdk::db::ISession> establish_mysql_session(
@@ -528,35 +611,53 @@ std::shared_ptr<mysqlshdk::db::ISession> establish_mysql_session(
   return establish_session(copy, prompt_for_password, prompt_in_loop);
 }
 
-void unpack_json_import_flags(shcore::Option_unpacker *unpacker,
-                              shcore::Document_reader_options *options) {
-  unpacker->optional("convertBsonTypes", &(*options).convert_bson_types);
+Connection_options get_classic_connection_options(
+    const std::shared_ptr<mysqlshdk::db::ISession> &session) {
+  // copy the connection options
+  auto co = session->get_connection_options();
+  // switch from X protocol to classic
+  if (SessionType::Classic != co.get_session_type()) {
+    co.clear_scheme();
+    co.set_scheme("mysql");
 
-  // Default value for convertBsonOid is the value of convertBsonTypes
-  options->convert_bson_id = options->convert_bson_types;
+    if (co.has_port()) {
+      const auto result = session->query("SELECT @@GLOBAL.port");
+      const auto row = result->fetch_one();
 
-  // convertBsonOid is independent, can be used even if convertBsonTypes is
-  // off
-  unpacker->optional("convertBsonOid", &(*options).convert_bson_id);
+      if (!row) {
+        throw std::logic_error(
+            "Unable to determine classic MySQL port from the given shell "
+            "connection");
+      }
 
-  // convertBsonOid unlocks extractOidTime
-  if (options->convert_bson_id)
-    unpacker->optional("extractOidTime", &(*options).extract_oid_time);
+      co.clear_port();
+      co.set_port(row->get_int(0));
 
-  // convertBsonTypes unlocks the rest of the options
-  if (options->convert_bson_types) {
-    unpacker->optional("ignoreDate", &(*options).ignore_date);
-    unpacker->optional("ignoreTimestamp", &(*options).ignore_timestamp);
-    unpacker->optional("ignoreBinary", &(*options).ignore_binary);
-    unpacker->optional("ignoreRegex", &(*options).ignore_regexp);
+      // if we're here, we clear up local port, so establish session will find
+      // the correct ssh tunnel or make a new one
+      co.get_ssh_options_handle().clear_local_port();
+    } else {
+      // if we're here then socket was used
+      const auto result =
+          session->query("SELECT @@GLOBAL.socket, @@GLOBAL.datadir");
+      const auto row = result->fetch_one();
 
-    // ignoreRegex locks ignoreRegexOptions
-    if (!options->ignore_regexp)
-      unpacker->optional("ignoreRegexOptions",
-                         &(*options).ignore_regexp_options);
+      if (!row) {
+        throw std::logic_error(
+            "Unable to determine classic MySQL socket from the given shell "
+            "connection");
+      }
 
-    unpacker->optional("decimalAsDouble", &(*options).decimal_as_double);
+      const auto socket = row->get_string(0);
+
+      co.clear_socket();
+      co.set_socket(shcore::path::is_absolute(socket)
+                        ? socket
+                        : shcore::path::join_path(row->get_string(1), socket));
+    }
   }
+
+  return co;
 }
 
 std::vector<shcore::Value> get_row_values(const mysqlshdk::db::IRow &row) {
@@ -602,7 +703,11 @@ std::vector<shcore::Value> get_row_values(const mysqlshdk::db::IRow &row) {
 
         case Type::Date:
         case Type::DateTime:
-          v = Value(Date::unrepr(row.get_string(i)));
+          v = Value::wrap(new Date(Date::unrepr(row.get_string(i))));
+          break;
+
+        case Type::Time:
+          v = Value::wrap(new Date(Date::unrepr(row.get_string(i))));
           break;
 
         case Type::Bit:
@@ -612,7 +717,6 @@ std::vector<shcore::Value> get_row_values(const mysqlshdk::db::IRow &row) {
         case Type::Bytes:
         case Type::Geometry:
         case Type::Json:
-        case Type::Time:
         case Type::Enum:
         case Type::Set:
           v = Value(row.get_string(i));
@@ -627,3 +731,22 @@ std::vector<shcore::Value> get_row_values(const mysqlshdk::db::IRow &row) {
 }
 
 }  // namespace mysqlsh
+
+// We need to hide these from doxygen to avoid warnings
+#if !defined DOXYGEN_JS && !defined DOXYGEN_PY
+namespace shcore {
+namespace detail {
+
+mysqlshdk::db::Connection_options Type_info<
+    mysqlshdk::db::Connection_options>::to_native(const shcore::Value &in) {
+  return mysqlsh::get_connection_options(in);
+}
+
+mysqlshdk::db::Connection_options
+Type_info<mysqlshdk::db::Connection_options>::default_value() {
+  return mysqlshdk::db::Connection_options();
+}
+
+}  // namespace detail
+}  // namespace shcore
+#endif
