@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -31,13 +31,6 @@ namespace import_table {
 
 constexpr const size_t BUFFER_SIZE = 1 << 16;
 
-namespace detail {
-/** Open file in read-only mode */
-int file_open(const std::string &pathname);
-
-/** Close file descriptor */
-void file_close(int fd);
-}  // namespace detail
 }  // namespace import_table
 }  // namespace mysqlsh
 
