@@ -97,8 +97,7 @@ Dump_reader::Status Dump_reader::open() {
   shcore::Dictionary_t basenames(md->get_map("basenames"));
 
   for (const auto &schema : *md->get_array("schemas")) {
-    if (m_options.include_schema(schema.as_string()) ||
-        m_options.include_table(schema.as_string(), "")) {
+    if (m_options.include_schema(schema.as_string())) {
       auto info = std::make_shared<Schema_info>();
 
       info->schema = schema.get_string();
@@ -262,12 +261,10 @@ std::vector<shcore::Account> Dump_reader::accounts() const {
 
   // parse the script to extract user list
   dump::Schema_dumper::preprocess_users_script(
-      script,
-      [&account_list](const std::string &account) {
+      script, [&account_list](const std::string &account) {
         account_list.emplace_back(shcore::split_account(account));
         return true;
-      },
-      {});
+      });
 
   return account_list;
 }
@@ -327,6 +324,12 @@ const std::vector<std::string> &Dump_reader::deferred_schema_fks(
   return s->fk_queries;
 }
 
+const std::vector<std::string> &Dump_reader::queries_on_schema_end(
+    const std::string &schema) const {
+  const auto &s = m_contents.schemas.at(schema);
+  return s->queries_on_schema_end;
+}
+
 const std::map<std::string, std::vector<std::string>>
 Dump_reader::tables_without_pk() const {
   std::map<std::string, std::vector<std::string>> res;
@@ -403,8 +406,8 @@ Dump_reader::schedule_chunk_proportionally(
 
     for (auto it = tables_with_data->begin(); it != tables_with_data->end();
          ++it) {
-      std::string key = partition_key((*it)->owner->schema, (*it)->owner->table,
-                                      (*it)->partition);
+      std::string key = schema_table_object_key(
+          (*it)->owner->schema, (*it)->owner->table, (*it)->partition);
       if (tables_being_loaded.find(key) == tables_being_loaded.end()) {
         if (best == tables_with_data->end() ||
             (*it)->bytes_available() > (*best)->bytes_available())
@@ -462,9 +465,9 @@ Dump_reader::schedule_chunk_proportionally(
       tables_with_data->begin();
 
   for (const auto &cand : candidate_weights) {
-    std::string key =
-        partition_key((*cand.first)->owner->schema, (*cand.first)->owner->table,
-                      (*cand.first)->partition);
+    std::string key = schema_table_object_key((*cand.first)->owner->schema,
+                                              (*cand.first)->owner->table,
+                                              (*cand.first)->partition);
     auto it = worker_weights.find(key);
     if (it != worker_weights.end()) {
       double d = cand.second - it->second;
@@ -541,7 +544,7 @@ bool Dump_reader::next_deferred_index(
 
         for (const auto &di : table.second->data_info) {
           keys.emplace_back(
-              partition_key(schema.first, table.first, di.partition));
+              schema_table_object_key(schema.first, table.first, di.partition));
         }
 
         if (load_finished(keys)) {
@@ -653,44 +656,28 @@ void Dump_reader::rescan(dump::Progress_thread *progress_thread) {
   compute_filtered_data_size();
 }
 
-uint64_t Dump_reader::add_deferred_indexes(const std::string &schema,
-                                           const std::string &table,
-                                           std::vector<std::string> &&indexes) {
-  auto s = m_contents.schemas.find(schema);
+uint64_t Dump_reader::add_deferred_statements(
+    const std::string &schema, const std::string &table,
+    compatibility::Deferred_statements &&stmts) {
+  const auto s = m_contents.schemas.find(schema);
   if (s == m_contents.schemas.end())
     throw std::runtime_error("Unable to find schema " + schema +
                              " for adding index");
-  auto t = s->second->tables.find(table);
+  const auto t = s->second->tables.find(table);
   if (t == s->second->tables.end())
     throw std::runtime_error("Unable to find table " + table + " in schema " +
                              schema + " for adding index");
-  t->second->indexes_done = false;
-  t->second->indexes = std::move(indexes);
-  auto &idx = t->second->indexes;
-  idx.erase(
-      std::remove_if(idx.begin(), idx.end(),
-                     [&s](const std::string &q) {
-                       mysqlshdk::utils::SQL_iterator it(q);
-                       while (it.valid() &&
-                              !shcore::str_caseeq(it.next_token(), "FOREIGN")) {
-                       }
-                       if (it.valid() &&
-                           shcore::str_caseeq(it.next_token(), "KEY")) {
-                         s->second->fk_queries.emplace_back(q);
-                         return true;
-                       }
-                       return false;
-                     }),
-      idx.end());
+  t->second->indexes_done = stmts.indexes.empty();
+  t->second->indexes = std::move(stmts.indexes);
+  std::move(stmts.fks.begin(), stmts.fks.end(),
+            std::back_inserter(s->second->fk_queries));
 
-  const auto size = idx.size();
-
-  if (0 == size) {
-    // if all queries were FK-related, we're done here
-    t->second->indexes_done = true;
+  if (!stmts.secondary_engine.empty()) {
+    s->second->queries_on_schema_end.emplace_back(
+        std::move(stmts.secondary_engine));
   }
 
-  return size;
+  return t->second->indexes.size();
 }
 
 void Dump_reader::replace_target_schema(const std::string &schema) {
@@ -842,6 +829,11 @@ void Dump_reader::Table_info::update_metadata(const std::string &data,
     primary_index = to_vector_of_strings(md->get_array("primaryIndex"));
   }
 
+  {
+    const auto trigger_list = md->get_array("triggers");
+    has_triggers = trigger_list && !trigger_list->empty();
+  }
+
   auto histogram_list = md->get_array("histograms");
 
   if (histogram_list) {
