@@ -56,14 +56,21 @@
 #define SHCORE_DEBUG "debug"
 #define SHCORE_MYSQL_PLUGIN_DIR "mysqlPluginDir"
 
+#define SHCORE_CONNECT_TIMEOUT "connectTimeout"
+#define SHCORE_DBA_CONNECT_TIMEOUT "dba.connectTimeout"
+
 #define SHCORE_PROGRESS_REPORTING "progressReporting"
 
 #include <stdlib.h>
+#include <array>
 #include <iostream>
 #include <memory>
+#include <optional>
 #include <string>
+#include <tuple>
 #include <vector>
 #include "mysqlshdk/libs/db/connection_options.h"
+#include "mysqlshdk/libs/ssh/ssh_connection_options.h"
 #include "mysqlshdk/libs/utils/options.h"
 #include "mysqlshdk/shellcore/shell_cli_operation.h"
 #include "shellcore/ishell_core.h"
@@ -73,6 +80,21 @@ namespace mysqlsh {
 class Shell_options : public shcore::Options {
  public:
   enum class Quiet_start { NOT_SET, SUPRESS_BANNER, SUPRESS_INFO };
+
+#ifdef _WIN32
+  using ssize_t = __int64;
+#endif  // _WIN32
+
+  struct Ssh_settings {
+    std::string known_hosts_file;
+    std::string identity_file;
+    std::string config_file;
+    int timeout = 10;
+    unsigned int buffer_size = 10240;
+    std::string uri;
+    std::string pwd;
+    mysqlshdk::ssh::Ssh_connection_options uri_data;
+  };
   struct Storage {
     shcore::IShell_core::Mode initial_mode = shcore::IShell_core::Mode::None;
     std::string run_file;
@@ -80,8 +102,6 @@ class Shell_options : public shcore::Options {
 
     // Individual connection parameters
     std::string user;
-    std::string pwd;
-    const char *password = nullptr;
     std::string host;
     int port = 0;
     std::string schema;
@@ -90,10 +110,12 @@ class Shell_options : public shcore::Options {
     std::string oci_profile;
     std::string oci_config_file;
     std::string auth_method;
-    std::string m_connect_timeout;
+    std::string connect_timeout_cmdline;
     std::string compress;
     std::string compress_algorithms;
+
     mysqlshdk::utils::nullable<int64_t> compress_level;
+    mysqlshdk::db::Mfa_passwords mfa_passwords;
 
     std::string protocol;
 
@@ -101,6 +123,7 @@ class Shell_options : public shcore::Options {
     mysqlshdk::db::Ssl_options ssl_options;
 
     std::string uri;
+    Ssh_settings ssh;
 
     std::string result_format;
     std::string wrap_json;
@@ -158,6 +181,9 @@ class Shell_options : public shcore::Options {
     mysqlshdk::null_string plugins_path;
     std::string mysql_plugin_dir;
 
+    double connect_timeout = 10.0;
+    double dba_connect_timeout = 5.0;
+
     // TODO(anyone): Expose the option
     enum class Progress_reporting {
       DISABLED,
@@ -171,6 +197,9 @@ class Shell_options : public shcore::Options {
 
     bool has_connection_data() const;
 
+    // Did user specify > 1 password?
+    bool is_mfa(bool check_values = false) const;
+
     mysqlshdk::db::Connection_options connection_options() const;
   };
 
@@ -248,6 +277,7 @@ class Shell_options : public shcore::Options {
   void check_port_socket_conflicts();
   void check_result_format();
   void check_file_execute_conflicts();
+  void check_ssh_conflicts();
 
   /**
    * --import option require default schema to be provided in connection
