@@ -399,7 +399,7 @@ bool get_group_information(const mysqlshdk::mysql::IInstance &instance,
   try {
     std::shared_ptr<db::IResult> result(instance.query(
         "SELECT @@group_replication_group_name group_name, "
-        "NULLIF(CONCAT(''/*!80025, @@group_replication_view_change_uuid*/), "
+        "NULLIF(CONCAT(''/*!80026, @@group_replication_view_change_uuid*/), "
         "'') group_view_change_uuid, "
         " @@group_replication_single_primary_mode single_primary, "
         " @@server_uuid, "
@@ -611,47 +611,6 @@ std::map<std::string, utils::nullable<std::string>> get_all_configurations(
   return gr_vars;
 }
 
-/**
- * Change the recovery user credentials for Group Replication.
- *
- * NOTE: Execute the CHANGE MASTER statement to configure the GR recovery user.
- *
- * @param instance session object to connect to the target instance.
- * @param rpl_user string with the username that will be used for replication.
- *                 Note: this user should already exist with the proper
- *                       privileges.
- * @param rpl_pwd string with the password for the replication user.
- */
-void change_recovery_credentials(const mysqlshdk::mysql::IInstance &instance,
-                                 const std::string &rpl_user,
-                                 const std::string &rpl_pwd) {
-  std::string source_term_cmd =
-      mysqlshdk::mysql::get_replication_source_keyword(instance.get_version(),
-                                                       true);
-
-  std::string source_term =
-      mysqlshdk::mysql::get_replication_source_keyword(instance.get_version());
-
-  std::string change_master_stmt_fmt =
-      "CHANGE " + source_term_cmd + " TO " + source_term +
-      "_USER = /*(*/ ? /*)*/, " + source_term +
-      "_PASSWORD = /*((*/ ? /*))*/ "
-      "FOR CHANNEL 'group_replication_recovery'";
-  shcore::sqlstring change_master_stmt =
-      shcore::sqlstring(change_master_stmt_fmt.c_str(), 0);
-  change_master_stmt << rpl_user;
-  change_master_stmt << rpl_pwd;
-  change_master_stmt.done();
-
-  try {
-    instance.execute(change_master_stmt);
-  } catch (const std::exception &err) {
-    throw std::runtime_error{
-        "Cannot set Group Replication recovery user to '" + rpl_user +
-        "'. Error executing CHANGE MASTER statement: " + err.what()};
-  }
-}
-
 /**
  * Start the Group Replication.
  *
@@ -795,32 +754,6 @@ mysqlshdk::mysql::Auth_options create_recovery_user(
   return creds;
 }
 
-/**
- * Get the replication user used for recovery.
- *
- * This function returns the replication user used in the (last)
- * CHANGE MASTER TO statement FOR CHANNEL 'group_replication_recovery'.
- *
- * NOTE: The correct execution of this function requires the variable
- *       master_info_repository=TABLE to be set which is a requirement for
- *       Group Replication.
- *
- * @param instance instance object of target member to obtain the
- *                 replication user.
- * @return a string with the replication (recovery) user set for the specified
- *         instance. Note: If no replication user was specified an empty string
- *         is returned.
- */
-std::string get_recovery_user(const mysqlshdk::mysql::IInstance &instance) {
-  std::string rpl_user;
-  std::shared_ptr<db::IResult> result(
-      instance.query("SELECT User_name FROM mysql.slave_master_info "
-                     "WHERE Channel_name = 'group_replication_recovery'"));
-  auto row = result->fetch_one();
-  if (row) rpl_user = row->get_string(0);
-  return rpl_user;
-}
-
 bool is_group_replication_delayed_starting(
     const mysqlshdk::mysql::IInstance &instance) {
   try {
@@ -949,52 +882,21 @@ void update_auto_increment(mysqlshdk::config::Config *config,
 }
 
 void update_group_seeds(mysqlshdk::config::Config *config,
-                        const std::string &gr_address,
-                        Gr_seeds_change_type change_type) {
+                        const std::map<std::string, std::string> &group_seeds) {
   std::vector<std::string> handler_names = config->list_handler_names();
 
   // Each instance might have a different group_seed value thus each handler is
   // set individually.
-  for (std::string handler_name : handler_names) {
-    utils::nullable<std::string> gr_group_seeds_new_value;
+  for (const std::string &handler_name : handler_names) {
+    std::string gr_group_seeds_new_value;
 
-    // Get the current group_seeds value.
-    mysqlshdk::utils::nullable<std::string> gr_group_seeds =
-        config->get_string("group_replication_group_seeds", handler_name);
-    auto gr_group_seeds_vector = shcore::split_string(*gr_group_seeds, ",");
-
-    // Determine the new value for the group_seeds.
-    switch (change_type) {
-      case Gr_seeds_change_type::ADD:
-        // Add the gr_address to the current group_seeds value.
-        if (!gr_group_seeds->empty()) {
-          // if the group_seeds value is not empty, add the gr_address to it.
-          // if it is not already there.
-          if (std::find(gr_group_seeds_vector.begin(),
-                        gr_group_seeds_vector.end(),
-                        gr_address) == gr_group_seeds_vector.end()) {
-            gr_group_seeds_vector.push_back(gr_address);
-          }
-          gr_group_seeds_new_value =
-              shcore::str_join(gr_group_seeds_vector, ",");
-        } else {
-          // If the instance had no group_seeds yet defined, just set it as the
-          // value the gr_address argument.
-          gr_group_seeds_new_value = gr_address;
-        }
-        break;
-      case Gr_seeds_change_type::REMOVE:
-        // Remove the gr_address from the current group_seeds value.
-        gr_group_seeds_vector.erase(
-            std::remove(gr_group_seeds_vector.begin(),
-                        gr_group_seeds_vector.end(), gr_address),
-            gr_group_seeds_vector.end());
-        gr_group_seeds_new_value = shcore::str_join(gr_group_seeds_vector, ",");
-        break;
-      case Gr_seeds_change_type::OVERRIDE:
-        // Override the group_seeds with the gr_address.
-        gr_group_seeds_new_value = gr_address;
-        break;
+    // assemble group_seeds list with all GR endpoints except their own
+    for (const auto &it : group_seeds) {
+      if (it.first != config->get_handler(handler_name)->get_server_uuid()) {
+        if (!gr_group_seeds_new_value.empty())
+          gr_group_seeds_new_value.append(",");
+        gr_group_seeds_new_value.append(it.second);
+      }
     }
 
     config->set_for_handler("group_replication_group_seeds",
