@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2017, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -79,25 +79,28 @@ std::string preprocess_markup(const std::string &line, Highlights *highlights) {
   std::string ret_val = line;
 
   // @code tags are only used in Doxygen, in the console they are removed.
-  size_t start = ret_val.find("@code");
-  size_t end = ret_val.find("@endcode", start);
-  while (start != std::string::npos && end != std::string::npos) {
-    std::string tmp_line = ret_val.substr(0, start);
-    size_t skip = 5;  // The length of @code
+  {
+    size_t start = ret_val.find("@code");
+    size_t end = ret_val.find("@endcode", start);
+    while (start != std::string::npos && end != std::string::npos) {
+      std::string tmp_line = ret_val.substr(0, start);
+      size_t skip = 5;  // The length of @code
 
-    tmp_line += ret_val.substr(start + skip, end - start - 5);
-    tmp_line += ret_val.substr(end + 8);
+      tmp_line += ret_val.substr(start + skip, end - start - 5);
+      tmp_line += ret_val.substr(end + 8);
 
-    ret_val = tmp_line;
-    start = ret_val.find("@code");
-    end = ret_val.find("@endcode", start);
+      ret_val = tmp_line;
+      start = ret_val.find("@code");
+      end = ret_val.find("@endcode", start);
+    }
   }
 
   // Some characters need to be specified using special doxygen format to
   // to prevent generating doxygen warnings.
   std::vector<std::pair<const char *, const char *>> replacements = {
-      {"@<", "<"}, {"@>", ">"}, {"&nbsp;", " "}, {"@li", "-"},
-      {"@%", "%"}, {"@@", "@"}, {"\\\\", "\\"},  {"<br>", "\n"}};
+      {"@<", "<"},   {"@>", ">"},     {"&nbsp;", " "}, {"@li", "-"},
+      {"@%", "%"},   {"@@", "@"},     {"\\\\", "\\"},  {"<br>", "\n"},
+      {"@::", "::"}, {"&quot;", "\""}};
 
   for (const auto &rpl : replacements) {
     ret_val = shcore::str_replace(ret_val, rpl.first, rpl.second);
@@ -589,7 +592,10 @@ std::string format_markup_text(const std::vector<std::string> &lines,
 
   if (!lines.empty()) {
     // Determine if we start with a list of items or not
-    bool in_list = (0 == lines[0].find("@li"));
+    int in_list = 0;
+    if (0 == lines[0].find("@li")) in_list = 1;
+    if (0 == lines[0].find("-# ")) in_list = 2;
+    int next_list_item = 1;
 
     std::string new_line;
     for (auto line : lines) {
@@ -620,16 +626,76 @@ std::string format_markup_text(const std::vector<std::string> &lines,
             format_markup_text(line.substr(4), width, left_padding + 2);
         formatted[left_padding] = '-';
 
-        if (!in_list) {
+        if (in_list != 1) {
           ret_val += "\n";
-          in_list = true;
+          in_list = 1;
         }
 
         ret_val += formatted;
+      } else if (0 == line.find("-# ")) {
+        // handles numbered list items:
+        // - Padding increases in 2 to let the item bullet alone.
+        // - Blank line is inserted before the first item.
+        std::string str_number = std::to_string(next_list_item++);
+        std::string formatted = format_markup_text(
+            line.substr(3), width, left_padding + 2 + str_number.size());
+
+        size_t index = 0;
+        for (index = 0; index < str_number.size(); index++) {
+          formatted[left_padding + index] = str_number[index];
+        }
+        formatted[left_padding + index] = '.';
+
+        if (in_list != 2) {
+          ret_val += "\n";
+          in_list = 2;
+        }
+
+        ret_val += formatted;
+      } else if (0 == line.find("@entry")) {
+        // a man-like entry item
+        static constexpr std::size_t indent = 12;
+        const auto append_newline = (0 == line.find("@entrynl"));
+
+        const auto begin = line.find("{");
+
+        if (std::string::npos == begin) {
+          throw std::invalid_argument("@entry has to be enclosed with {}");
+        }
+
+        const auto end = line.find("}");
+
+        if (std::string::npos == end) {
+          throw std::invalid_argument("@entry has to be enclosed with {}");
+        }
+
+        const auto entry_length = end - begin - 1;
+        const auto entry = format_markup_text(
+            "<b>" + line.substr(begin + 1, entry_length) + "</b>", width,
+            left_padding);
+
+        const auto desc = line.find_first_not_of(" \t\r\n", end + 1);
+
+        auto description = format_markup_text(
+            std::string::npos == desc ? "" : line.substr(desc), width,
+            left_padding + indent);
+
+        if (in_list != 1 || append_newline) {
+          ret_val += "\n";
+          in_list = 1;
+        }
+
+        if (entry_length > indent - 1) {
+          ret_val += entry + "\n";
+          ret_val += description;
+        } else {
+          description.replace(0, left_padding + entry_length, entry);
+          ret_val += description;
+        }
       } else {
         if ((!ret_val.empty() && paragraph_per_line) || in_list) {
           ret_val += "\n";
-          in_list = false;
+          in_list = 0;
         }
 
         if (0 == line.find("@warning ")) {
