@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2020, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -22,7 +22,10 @@
  */
 
 #include "mysqlshdk/libs/utils/utils_mysql_parsing.h"
+#include <algorithm>
+#include <iostream>
 #include <iterator>
+#include <sstream>
 #include <tuple>
 #include <utility>
 #include "mysqlshdk/libs/utils/utils_lexing.h"
@@ -31,13 +34,25 @@
 namespace mysqlshdk {
 namespace utils {
 
+Sql_splitter::Sql_splitter(const Command_callback &cmd_callback,
+                           const Error_callback &err_callback,
+                           const std::initializer_list<const char *> &commands)
+    : m_cmd_callback(cmd_callback), m_err_callback(err_callback) {
+  for (const char *cmd : commands) {
+    size_t off = tolower(cmd[0]) - 'a';
+    assert(off < m_commands_table.size());
+    m_commands_table[off].emplace_front(cmd);
+  }
+  reset();
+}
+
 void Sql_splitter::reset() {
   m_begin = nullptr;
   m_end = nullptr;
   m_ptr = nullptr;
 
   m_delimiter = ";";
-  m_context = NONE;
+  m_context.clear();
 
   m_shrinked_bytes = 0;
   m_current_line = 1;
@@ -75,7 +90,7 @@ void Sql_splitter::feed_chunk(char *buffer, size_t size) {
 
   m_ptr = m_begin;
 
-  m_context = NONE;
+  m_context.clear();
 }
 
 void Sql_splitter::set_ansi_quotes(bool flag) { m_ansi_quotes = flag; }
@@ -94,7 +109,7 @@ bool Sql_splitter::set_delimiter(const std::string &delim) {
   return true;
 }
 
-template <const char skip_table[255], const char quote>
+template <const char skip_table[256], const char quote>
 inline char *span_string(char *p, const char *end) {
   // p must be inside the single quote string (after the ')
   int last_ch = 0;
@@ -194,16 +209,15 @@ constexpr int k_delimiter_len = 9;
  */
 bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
                               std::string *out_delim) {
-  auto unfinished_stmt = [this](char *bos, Range *out_range) {
+  auto unfinished_stmt = [this](char *bos, Range *result_range) {
     Range range{static_cast<size_t>(bos - m_begin),
                 static_cast<size_t>(m_end - bos), m_current_line};
     m_ptr = bos;
-    *out_range = range;
+    *result_range = range;
     return false;
   };
 
   char *bos;  // beginning of statement
-  Context previous_context = Context::NONE;
 
   bos = m_ptr;
   for (;;) {
@@ -221,6 +235,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
   while (next_bol < m_end) {
     char *bol = next_bol;
     bool has_complete_line;
+    bool command = false;
     bool last_line_was_delimiter = false;
     // process input per line so we can count line numbers
     char *real_eol;
@@ -236,26 +251,58 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
 
     // find the end of the statement
     char *p = bol;
+
+    const auto run_command = [&](bool delimited) {
+      command = false;
+      size_t skip;
+      bool d;
+      std::tie(skip, d) = m_cmd_callback(bos, p - bos, true, m_current_line);
+      if (skip == 0) return false;
+      if (delimited) skip += m_delimiter.size();
+      p = bos + skip;
+      memmove(bos, p, m_end - p + 1);
+      m_shrinked_bytes += skip;
+      p = bos;
+      eol -= skip;
+      next_bol -= skip;
+      m_end -= skip;
+      pop();
+      return true;
+    };
+
     while (p && p < eol) {
-      if (m_context == NONE || m_context == STATEMENT) {
-        // check for the current delimiter
-        if (*p == m_delimiter[0] &&
-            (p + m_delimiter.size() <= eol &&
-             m_delimiter.compare(0, m_delimiter.size(), p, 0,
-                                 m_delimiter.size()) == 0)) {
-          m_context = NONE;
-          Range range{static_cast<size_t>(bos - m_begin),
-                      static_cast<size_t>(p - bos), m_current_line};
-          m_ptr = p + m_delimiter.size();
-          *out_range = range;
-          *out_delim = m_delimiter;
-          m_current_line += line_count;
-          return true;
+      auto ctx = context();
+      if (ctx == NONE || ctx == STATEMENT || ctx == COMMENT_CONDITIONAL) {
+        if (ctx == COMMENT_CONDITIONAL) {
+          if ((eol - p) > 2 && *p == '*' && *(p + 1) == '/') {
+            pop();
+            p += 2;
+            continue;
+          }
+        } else {
+          // check for the current delimiter
+          if (*p == m_delimiter[0] &&
+              (p + m_delimiter.size() <= eol &&
+               m_delimiter.compare(0, m_delimiter.size(), p, 0,
+                                   m_delimiter.size()) == 0)) {
+            if (command && run_command(true)) continue;
+            pop();
+            Range range{static_cast<size_t>(bos - m_begin),
+                        static_cast<size_t>(p - bos), m_current_line};
+            m_ptr = p + m_delimiter.size();
+            *out_range = range;
+            *out_delim = m_delimiter;
+            m_current_line += line_count;
+            return true;
+          }
         }
 
         switch (*p) {
           case '\\':  // \x
           {
+            // commands within commands are not supported
+            if (command) goto other;
+
             // let the caller handle \commands
             size_t skip;
             bool delim;
@@ -281,7 +328,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
               m_ptr = p + skip;
               *out_range = Range{static_cast<size_t>(bos - m_begin),
                                  static_cast<size_t>(p - bos), m_current_line};
-              m_context = NONE;
+              pop();
               m_current_line += line_count;
               return true;
             } else {
@@ -296,38 +343,40 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
 
           case '\'':  // 'str'
             ++p;      // skip the opening '
-            m_context = SQUOTE_STRING;
+            push(SQUOTE_STRING);
             break;
 
           case '"':  // "str"
             ++p;     // skip the opening "
             if (m_ansi_quotes) {
-              m_context = DQUOTE_IDENTIFIER;
+              push(DQUOTE_IDENTIFIER);
             } else {
-              m_context = DQUOTE_STRING;
+              push(DQUOTE_STRING);
             }
             break;
 
           case '`':  // `ident`
             ++p;
-            m_context = BQUOTE_IDENTIFIER;
+            push(BQUOTE_IDENTIFIER);
             break;
 
           case '/':  // /* ... */
             if ((m_end - p) >= 3) {
               if (*(p + 1) == '*') {
-                previous_context = m_context;
-                if (*(p + 2) == '!' || *(p + 2) == '+') {
-                  m_context = COMMENT_HINT;
+                if (*(p + 2) == '+') {
+                  push(COMMENT_HINT);
+                  p += 3;
+                } else if (*(p + 2) == '!') {
+                  push(COMMENT_CONDITIONAL);
                   p += 3;
                 } else {
-                  m_context = COMMENT;
+                  push(COMMENT);
                   p += 2;
                 }
                 break;
               }
             } else {
-              return unfinished_stmt(bos, out_range);
+              if (!has_complete_line) return unfinished_stmt(bos, out_range);
             }
             goto other;
 
@@ -335,7 +384,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
           case 'D':
             // Possible start of the keyword DELIMITER. Must be the 1st keyword
             // of a statement.
-            if (m_context == NONE && (m_end - p >= k_delimiter_len) &&
+            if (context() == NONE && (m_end - p >= k_delimiter_len) &&
                 shcore::str_caseeq(p, k_delimiter, k_delimiter_len)) {
               if (has_complete_line) {
                 // handle delimiter change directly here
@@ -348,7 +397,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
                   set_delimiter(std::string(p, end - p));
                 }
                 bos = p = next_bol;
-                m_context = NONE;
+                pop();
                 last_line_was_delimiter = true;
                 // delimiter is like a full statement, so increment line
                 // and start next stmt from scratch
@@ -364,7 +413,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
           case '#':  // # ...
             if (has_complete_line) {
               // if the whole line is a comment return it
-              if (m_context == NONE) {
+              if (context() == NONE) {
                 int nl = 1;
                 if (*(eol - 1) == '\r') nl++;
                 Range range{static_cast<size_t>(bos - m_begin),
@@ -387,7 +436,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
             if ((m_end - p) > 2 && *(p + 1) == '-' && is_any_blank(*(p + 2))) {
               if (has_complete_line) {
                 // if the whole line is a comment return it
-                if (m_context == NONE) {
+                if (context() == NONE) {
                   int nl = 1;
                   if (*(eol - 1) == '\r') nl++;
                   Range range{static_cast<size_t>(bos - m_begin),
@@ -411,18 +460,37 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
 
           default:
           other:
-            if (m_context != NONE || !is_any_blank(*p)) m_context = STATEMENT;
-            if (p == bos && is_any_blank(*p)) bos++;
+            if (context() == NONE) {
+              size_t off = tolower(*p) - 'a';
+              if (off < m_commands_table.size() &&
+                  !m_commands_table[off].empty()) {
+                for (const auto &kwd : m_commands_table[off])
+                  if ((m_end - p >= static_cast<int>(kwd.length())) &&
+                      shcore::str_caseeq(p, kwd.c_str(), kwd.length()) &&
+                      is_any_blank(*(p + kwd.length()))) {
+                    command = true;
+                    p += kwd.length() + 1;
+                    push(STATEMENT);
+                    break;
+                  }
+                if (command) continue;
+              }
+            }
+
+            if (!is_any_blank(*p)) {
+              if (context() == NONE) push(STATEMENT);
+            } else if (p == bos) {
+              bos++;
+            }
             ++p;
             break;
         }
       }
 
-      switch (m_context) {
+      switch (context()) {
         case NONE:
-          break;
-
         case STATEMENT:
+        case COMMENT_CONDITIONAL:
           break;
 
         case SQUOTE_STRING:
@@ -430,13 +498,12 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
                                                                          eol);
           if (!p) {  // closing quote missing
             if (has_complete_line) {
-              m_context = SQUOTE_STRING;
               p = eol;
             } else {
               return unfinished_stmt(bos, out_range);
             }
           } else {
-            m_context = STATEMENT;
+            pop();
           }
           break;
 
@@ -444,13 +511,12 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
           p = span_string<internal::k_quoted_string_span_skips_dq, '"'>(p, eol);
           if (!p) {  // closing quote missing
             if (has_complete_line) {
-              m_context = DQUOTE_STRING;
               p = eol;
             } else {
               return unfinished_stmt(bos, out_range);
             }
           } else {
-            m_context = STATEMENT;
+            pop();
           }
           break;
 
@@ -465,7 +531,7 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
             }
           } else {
             p += 2;
-            m_context = previous_context;
+            pop();
           }
           break;
 
@@ -473,13 +539,12 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
           p = span_quoted_identifier<'`'>(p, eol);
           if (!p) {  // closing quote missing
             if (has_complete_line) {
-              m_context = BQUOTE_IDENTIFIER;
               p = eol;
             } else {
               return unfinished_stmt(bos, out_range);
             }
           } else {
-            m_context = STATEMENT;
+            pop();
           }
           break;
 
@@ -487,19 +552,23 @@ bool Sql_splitter::next_range(Sql_splitter::Range *out_range,
           p = span_quoted_identifier<'"'>(p, eol);
           if (!p) {  // closing quote missing
             if (has_complete_line) {
-              m_context = DQUOTE_IDENTIFIER;
               p = eol;
             } else {
               return unfinished_stmt(bos, out_range);
             }
           } else {
-            m_context = STATEMENT;
+            pop();
           }
           break;
       }
     }
 
-    if (p == eol && !last_line_was_delimiter) line_count++;
+    if (p == eol) {
+      if (command)
+        run_command(false);
+      else if (!last_line_was_delimiter)
+        line_count++;
+    }
   }
   if (m_last_chunk && bos < m_end) {
     Range range{static_cast<size_t>(bos - m_begin),
@@ -532,16 +601,30 @@ std::vector<std::tuple<std::string, std::string, size_t>> split_sql_stream(
     const Sql_splitter::Error_callback &err_callback, bool ansi_quotes,
     std::string *delimiter) {
   std::vector<std::tuple<std::string, std::string, size_t>> results;
-  iterate_sql_stream(stream, chunk_size,
-                     [&results](const char *s, size_t len,
-                                const std::string &delim, size_t lnum) {
-                       results.emplace_back(std::string(s, len), delim, lnum);
-                       return true;
-                     },
-                     err_callback, ansi_quotes, delimiter);
+  iterate_sql_stream(
+      stream, chunk_size,
+      [&results](const char *s, size_t len, const std::string &delim,
+                 size_t lnum) {
+        results.emplace_back(std::string(s, len), delim, lnum);
+        return true;
+      },
+      err_callback, ansi_quotes, delimiter);
   return results;
 }
 
+std::vector<std::string> split_sql(const std::string &str) {
+  std::istringstream s(str);
+  auto parts = split_sql_stream(&s, str.size(), [](const std::string &err) {
+    throw std::runtime_error("Error splitting SQL script: " + err);
+  });
+
+  std::vector<std::string> stmts;
+  for (const auto &p : parts) {
+    stmts.emplace_back(std::get<0>(p) + std::get<1>(p));
+  }
+  return stmts;
+}
+
 /** Apply a callback on each statement read from a stream object.
  *
  * @param stream stream object to read from
@@ -577,7 +660,7 @@ bool iterate_sql_stream(
         }
         return std::make_pair(2U, true);
       },
-      err_callback);
+      err_callback, {"source"});
   splitter.set_ansi_quotes(ansi_quotes);
   if (delimiter) splitter.set_delimiter(*delimiter);
   if (splitter_ptr) *splitter_ptr = &splitter;
@@ -632,7 +715,7 @@ bool iterate_sql_stream(
   if (delimiter) *delimiter = splitter.delimiter();
 
   return !stop;
-}  // namespace utils
+}
 
 std::string to_string(Sql_splitter::Context context) {
   switch (context) {
@@ -640,6 +723,7 @@ std::string to_string(Sql_splitter::Context context) {
       return "";
     case Sql_splitter::STATEMENT:
     case Sql_splitter::COMMENT_HINT:
+    case Sql_splitter::COMMENT_CONDITIONAL:
       return "-";
     case Sql_splitter::COMMENT:
       return "/*";
