@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2021, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -28,14 +28,16 @@
 #include <cstdarg>
 #include <cstdio>
 #include <cstring>
+#include <iostream>
 #include <limits>
 #include <locale>
 #include <sstream>
 #include <stdexcept>
+#include "mysqlshdk/libs/utils/dtoa.h"
 #include "mysqlshdk/libs/utils/logger.h"
-#include "utils/dtoa.h"
-#include "utils/utils_general.h"
-#include "utils/utils_string.h"
+#include "mysqlshdk/libs/utils/utils_general.h"
+#include "mysqlshdk/libs/utils/utils_json.h"
+#include "mysqlshdk/libs/utils/utils_string.h"
 
 #ifdef WIN32
 #ifdef max
@@ -43,6 +45,10 @@
 #endif
 #endif
 
+namespace shcore {
+
+namespace {
+
 // These is* functions have undefined behavior if the passed value
 // is out of the -1-255 range
 #define IS_ALPHA(x) (isalpha(static_cast<unsigned char>(x)))
@@ -52,7 +58,7 @@
 // from_type = row, to_type = column
 #define T true
 #define F false
-const bool shcore::kTypeConvertible[12][12] = {
+const bool kTypeConvertible[12][12] = {
     // Undf, Null,Bool,Str, Int, UInt,Flot,Obj, Arr, Map, MapR,Fun
     {T, F, F, F, F, F, F, F, F, F, F, F},  // Undefined
     {T, T, F, F, F, F, F, T, T, T, T, T},  // Null
@@ -91,10 +97,6 @@ static const uint32_t ascii_to_hex[256] = {
     0,  0,  0,  0, 0, 0,  0,  0,  0,  0,  0,  0, 0, 0, 0, 0, 0, 0,  0,  0,
     0,  0,  0,  0, 0, 0,  0,  0,  0,  0,  0,  0, 0, 0, 0};
 
-namespace shcore {
-
-namespace {
-
 void skip_whitespace(const char **pc) {
   while (**pc == ' ' || **pc == '\t' || **pc == '\r' || **pc == '\n' ||
          **pc == '\v' || **pc == '\f')
@@ -105,18 +107,28 @@ void skip_whitespace(const char **pc) {
 
 // --
 
-Exception::Exception(const std::shared_ptr<Value::Map_type> e) : _error(e) {}
+Exception::Exception(const std::string &message, int code,
+                     const std::shared_ptr<Value::Map_type> &e)
+    : shcore::Error(message, code), _error(e) {}
 
-Exception::Exception(const std::string &message, int code) {
+Exception::Exception(const std::string &message, int code)
+    : shcore::Error(message, code) {
   _error = shcore::make_dict("type", Value("MYSQLSH"), "message",
                              Value(message), "code", Value(code));
 }
 
+Exception::Exception(const std::string &type, const std::string &message,
+                     int code)
+    : shcore::Error(message, code) {
+  _error = shcore::make_dict("type", Value(type), "message", Value(message),
+                             "code", Value(code));
+}
+
 Exception Exception::argument_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("ArgumentError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -124,7 +136,7 @@ Exception Exception::attrib_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("AttributeError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -132,7 +144,7 @@ Exception Exception::type_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("TypeError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -140,7 +152,7 @@ Exception Exception::value_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("ValueError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -148,7 +160,7 @@ Exception Exception::logic_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("LogicError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -156,7 +168,7 @@ Exception Exception::runtime_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("RuntimeError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -164,7 +176,7 @@ Exception Exception::scripting_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("ScriptingError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -172,7 +184,7 @@ Exception Exception::metadata_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("MetadataError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, -1, error);
   return e;
 }
 
@@ -182,7 +194,7 @@ Exception Exception::error_with_code(const std::string &type,
   (*error)["type"] = Value(type);
   (*error)["message"] = Value(message);
   (*error)["code"] = Value(code);
-  Exception e(error);
+  Exception e(message, code, error);
   return e;
 }
 
@@ -194,7 +206,7 @@ Exception Exception::error_with_code_and_state(const std::string &type,
   (*error)["message"] = Value(message);
   (*error)["code"] = Value(code);
   if (sqlstate && *sqlstate) (*error)["state"] = Value(std::string(sqlstate));
-  Exception e(error);
+  Exception e(message, code, error);
   return e;
 }
 
@@ -202,7 +214,7 @@ Exception Exception::parser_error(const std::string &message) {
   std::shared_ptr<Value::Map_type> error(new Value::Map_type());
   (*error)["type"] = Value("ParserError");
   (*error)["message"] = Value(message);
-  Exception e(error);
+  Exception e(message, 0, error);
   return e;
 }
 
@@ -211,29 +223,12 @@ void Exception::set_file_context(const std::string &file, size_t line) {
   if (line > 0) (*_error)["line"] = Value(static_cast<uint64_t>(line));
 }
 
-const char *Exception::what() const noexcept {
-  if ((*_error)["message"].type == String)
-    return (*_error)["message"].value.s->c_str();
-  return "?";
-}
-
 const char *Exception::type() const noexcept {
   if ((*_error)["type"].type == String)
     return (*_error)["type"].value.s->c_str();
   return "Exception";
 }
 
-int Exception::code() const noexcept {
-  if ((*_error).find("code") != (*_error).end()) {
-    try {
-      return static_cast<int>((*_error)["code"].as_int());
-    } catch (...) {
-      return 0;  // as it's not int
-    }
-  }
-  return 0;
-}
-
 bool Exception::is_argument() const {
   return strcmp(type(), "ArgumentError") == 0;
 }
@@ -246,9 +241,19 @@ bool Exception::is_value() const { return strcmp(type(), "ValueError") == 0; }
 
 bool Exception::is_type() const { return strcmp(type(), "TypeError") == 0; }
 
+bool Exception::is_runtime() const {
+  return strcmp(type(), "RuntimeError") == 0;
+}
+
+bool Exception::is_metadata() const {
+  return strcmp(type(), "MetadataError") == 0;
+}
+
 bool Exception::is_mysql() const { return strcmp(type(), "MySQL Error") == 0; }
 
-bool Exception::is_mysqlsh() const { return strcmp(type(), "MYSQLSH") == 0; }
+bool Exception::is_mysqlsh() const {
+  return strcmp(type(), "MySQL Shell Error") == 0;
+}
 
 bool Exception::is_parser() const { return strcmp(type(), "ParserError") == 0; }
 
@@ -256,20 +261,16 @@ std::string Exception::format() const {
   std::string error_message;
 
   std::string type = _error->get_string("type", "");
-  std::string message = _error->get_string("message", "");
-  int64_t code = _error->get_int("code", -1);
+  std::string message = what();
   std::string state = _error->get_string("state", "");
   std::string error_location = _error->get_string("location", "");
 
-  if (!message.empty() &&
-      !is_mysqlsh()) {  // don't show shell error codes for now
+  if (!message.empty()) {
     if (!type.empty()) error_message += type;
 
-    if (code != -1) {
-      if (is_mysql() || is_mysqlsh())
-        error_message += "-" + std::to_string(code);
-      else
-        error_message += " " + std::to_string(code);
+    if (code() != -1 &&
+        !is_mysqlsh()) {  // don't show shell error codes for now
+      error_message += " " + std::to_string(code());
       if (!state.empty()) error_message += " (" + state + ")";
     }
 
@@ -278,8 +279,6 @@ std::string Exception::format() const {
     error_message += message;
 
     if (!error_location.empty()) error_message += " at " + error_location;
-
-    error_message += "\n";
   }
 
   return error_message;
@@ -424,10 +423,18 @@ void Value::Map_type::merge_contents(std::shared_ptr<Map_type> source,
 
 Value::Value(const Value &copy) : type(shcore::Null) { operator=(copy); }
 
+Value::Value(Value &&other) : type(shcore::Null) {
+  operator=(std::move(other));
+}
+
 Value::Value(const std::string &s) : type(String) {
   value.s = new std::string(s);
 }
 
+Value::Value(std::string &&s) : type(String) {
+  value.s = new std::string(std::move(s));
+}
+
 Value::Value(const char *s) {
   if (s) {
     type = String;
@@ -458,16 +465,14 @@ Value::Value(float f) : type(Float) {
   // direct typecast from float to double works by just appending 0s to the
   // binary IEEE representation, which will result in a different number
   // So we convert through decimal instead
-  char buffer[100];
-  my_gcvt(f, MY_GCVT_ARG_FLOAT, sizeof(buffer) - 1, buffer, NULL);
-  value.d = std::stod(buffer);
+  value.d = std::stod(shcore::ftoa(f));
 }
 
 Value::Value(double d) : type(Float) { value.d = d; }
 
 Value::Value(bool b) : type(Bool) { value.b = b; }
 
-Value::Value(std::shared_ptr<Function_base> f) : type(Function) {
+Value::Value(const std::shared_ptr<Function_base> &f) : type(Function) {
   if (f) {
     value.func = new std::shared_ptr<Function_base>(f);
   } else {
@@ -475,7 +480,15 @@ Value::Value(std::shared_ptr<Function_base> f) : type(Function) {
   }
 }
 
-Value::Value(std::shared_ptr<Object_bridge> n) : type(Object) {
+Value::Value(std::shared_ptr<Function_base> &&f) : type(Function) {
+  if (f) {
+    value.func = new std::shared_ptr<Function_base>(std::move(f));
+  } else {
+    type = shcore::Null;
+  }
+}
+
+Value::Value(const std::shared_ptr<Object_bridge> &n) : type(Object) {
   if (n) {
     value.o = new std::shared_ptr<Object_bridge>(n);
   } else {
@@ -483,7 +496,15 @@ Value::Value(std::shared_ptr<Object_bridge> n) : type(Object) {
   }
 }
 
-Value::Value(Map_type_ref n) : type(Map) {
+Value::Value(std::shared_ptr<Object_bridge> &&n) : type(Object) {
+  if (n) {
+    value.o = new std::shared_ptr<Object_bridge>(std::move(n));
+  } else {
+    type = shcore::Null;
+  }
+}
+
+Value::Value(const Map_type_ref &n) : type(Map) {
   if (n) {
     value.map = new std::shared_ptr<Map_type>(n);
   } else {
@@ -491,11 +512,23 @@ Value::Value(Map_type_ref n) : type(Map) {
   }
 }
 
-Value::Value(std::weak_ptr<Map_type> n) : type(MapRef) {
+Value::Value(Map_type_ref &&n) : type(Map) {
+  if (n) {
+    value.map = new std::shared_ptr<Map_type>(std::move(n));
+  } else {
+    type = shcore::Null;
+  }
+}
+
+Value::Value(const std::weak_ptr<Map_type> &n) : type(MapRef) {
   value.mapref = new std::weak_ptr<Map_type>(n);
 }
 
-Value::Value(Array_type_ref n) : type(Array) {
+Value::Value(std::weak_ptr<Map_type> &&n) : type(MapRef) {
+  value.mapref = new std::weak_ptr<Map_type>(std::move(n));
+}
+
+Value::Value(const Array_type_ref &n) : type(Array) {
   if (n) {
     value.array = new std::shared_ptr<Array_type>(n);
   } else {
@@ -503,6 +536,14 @@ Value::Value(Array_type_ref n) : type(Array) {
   }
 }
 
+Value::Value(Array_type_ref &&n) : type(Array) {
+  if (n) {
+    value.array = new std::shared_ptr<Array_type>(std::move(n));
+  } else {
+    type = shcore::Null;
+  }
+}
+
 Value &Value::operator=(const Value &other) {
   if (type == other.type) {
     switch (type) {
@@ -609,6 +650,83 @@ Value &Value::operator=(const Value &other) {
   return *this;
 }
 
+Value &Value::operator=(Value &&other) {
+  switch (type) {
+    case Undefined:
+    case shcore::Null:
+    case Bool:
+    case Integer:
+    case UInteger:
+    case Float:
+      break;
+    case String:
+      delete value.s;
+      value.s = nullptr;
+      break;
+    case Object:
+      delete value.o;
+      value.s = nullptr;
+      break;
+    case Array:
+      delete value.array;
+      value.s = nullptr;
+      break;
+    case Map:
+      delete value.map;
+      value.s = nullptr;
+      break;
+    case MapRef:
+      delete value.mapref;
+      value.s = nullptr;
+      break;
+    case Function:
+      delete value.func;
+      value.s = nullptr;
+      break;
+  }
+
+  type = Undefined;
+  std::swap(type, other.type);
+
+  switch (type) {
+    case Undefined:
+      break;
+    case shcore::Null:
+      break;
+    case Bool:
+      std::swap(value.b, other.value.b);
+      break;
+    case Integer:
+      std::swap(value.i, other.value.i);
+      break;
+    case UInteger:
+      std::swap(value.ui, other.value.ui);
+      break;
+    case Float:
+      std::swap(value.d, other.value.d);
+      break;
+    case String:
+      std::swap(value.s, other.value.s);
+      break;
+    case Object:
+      std::swap(value.o, other.value.o);
+      break;
+    case Array:
+      std::swap(value.array, other.value.array);
+      break;
+    case Map:
+      std::swap(value.map, other.value.map);
+      break;
+    case MapRef:
+      std::swap(value.mapref, other.value.mapref);
+      break;
+    case Function:
+      std::swap(value.func, other.value.func);
+      break;
+  }
+  return *this;
+}
+
 Value Value::parse_map(const char **pc) {
   Map_type_ref map(new Map_type());
 
@@ -832,36 +950,63 @@ Value Value::parse_number(const char **pcc) {
   Value ret_val;
   const char *pc = *pcc;
 
-  // Sign can appear at the beggining
-  if (*pc == '-' || *pc == '+') ++pc;
+  // Parsing based on the following state table
+  // Valid integer: state == 1
+  // Valid float: state == 2,4,6
+  //                                0  1  2  3  4  5  6
+  //                                +  #  .  #  e  +  d
+  constexpr unsigned char STATE_VALIDITY[] = {0, 1, 0, 1, 0, 0, 1};
+  enum States {
+    FRONT_SIGN,
+    INT_DIGITS,
+    DOT,
+    FLOAT_DIGITS,
+    EXP,
+    EXP_SIGN,
+    EXP_DIGITS
+  };
 
-  // Continues while there are digits
-  while (*pc && IS_DIGIT(*++pc)) {
+  // State starts at 1
+  int state = INT_DIGITS;
+
+  // Sign can appear at the beggining
+  if (*pc == '-' || *pc == '+') {
+    ++pc;
+    state = FRONT_SIGN;
   }
 
-  bool is_integer = true;
-  if (tolower(*pc) == '.') {
-    is_integer = false;
+  // Continues while there are digits
+  if (*pc && IS_DIGIT(*pc)) state = INT_DIGITS;
+  while (*pc && IS_DIGIT(*++pc))
+    ;
+
+  if (STATE_VALIDITY[state] == 1 && tolower(*pc) == '.') {
+    state = DOT;
 
     // Skips the .
     ++pc;
 
     // Continues while there are digits
-    while (*pc && IS_DIGIT(*++pc)) {
-    }
+    if (*pc && IS_DIGIT(*pc)) state = FLOAT_DIGITS;
+    while (*pc && IS_DIGIT(*++pc))
+      ;
   }
 
-  if (tolower(*pc) == 'e')  // exponential
-  {
-    is_integer = false;
+  // exponential
+  if (STATE_VALIDITY[state] == 1 && tolower(*pc) == 'e') {
+    state = EXP;
 
     // Skips the e
     ++pc;
 
     // Sign can appear for exponential numbers
-    if (*pc == '-' || *pc == '+') ++pc;
+    if (*pc == '-' || *pc == '+') {
+      ++pc;
+      state = EXP_SIGN;
+    }
 
     // Continues while there are digits
+    if (*pc && IS_DIGIT(*pc)) state = FLOAT_DIGITS;
     while (*pc && IS_DIGIT(*++pc))
       ;
   }
@@ -869,27 +1014,33 @@ Value Value::parse_number(const char **pcc) {
   size_t len = pc - *pcc;
   std::string number(*pcc, len);
 
-  if (is_integer) {
-    int64_t ll = 0;
-    try {
-      ll = std::atoll(number.c_str());
-    } catch (const std::invalid_argument &e) {
-      std::string s = "Error parsing int: ";
-      s += e.what();
-      throw Exception::parser_error(s);
-    }
-    ret_val = Value(static_cast<int64_t>(ll));
-  } else {
-    double d = 0;
-    try {
-      d = std::stod(number.c_str());
-    } catch (const std::invalid_argument &e) {
-      std::string s = "Error parsing float: ";
-      s += e.what();
-      throw Exception::parser_error(s);
-    }
+  if (STATE_VALIDITY[state] == 1) {
+    if (state == INT_DIGITS) {
+      int64_t ll = 0;
+      try {
+        ll = std::atoll(number.c_str());
+      } catch (const std::invalid_argument &e) {
+        std::string s = "Error parsing int: ";
+        s += e.what();
+        throw Exception::parser_error(s);
+      }
+      ret_val = Value(static_cast<int64_t>(ll));
+    } else {
+      double d = 0;
+      try {
+        d = std::stod(number.c_str());
+      } catch (const std::invalid_argument &e) {
+        std::string s = "Error parsing float: ";
+        s += e.what();
+        throw Exception::parser_error(s);
+      }
 
-    ret_val = Value(d);
+      ret_val = Value(d);
+    }
+  } else {
+    std::string parsed(*pcc, pc - *pcc);
+    throw Exception::parser_error("Error parsing number from: '" + parsed +
+                                  "'");
   }
 
   *pcc = pc;
@@ -919,6 +1070,29 @@ Value Value::parse(const std::string &s) {
   return tmp;
 }
 
+Value Value::parse(const char *s, size_t size) {
+  const char *begin = s;
+  const char *pc = begin;
+
+  // ignore any white-space at the beginning of string
+  skip_whitespace(&pc);
+
+  Value tmp(parse(&pc));
+  size_t parsed_length = pc - begin;
+
+  if (parsed_length < size) {
+    // ensure any leftover chars are just whitespaces
+    skip_whitespace(&pc);
+    parsed_length = pc - begin;
+    if (parsed_length < size)
+      throw Exception::parser_error(
+          "Unexpected characters left at the end of document: ..." +
+          std::string(pc));
+  }
+
+  return tmp;
+}
+
 Value Value::parse(const char **pc) {
   if (**pc == '{') {
     return parse_map(pc);
@@ -1019,8 +1193,8 @@ bool Value::operator==(const Value &other) const {
           case Bool:
             return other.operator==(*this);
           case Float:
-            return value.i == (int)other.value.d &&
-                   ((other.value.d - (int)other.value.d) == 0.0);
+            return value.i == (int64_t)other.value.d &&
+                   ((other.value.d - (int64_t)other.value.d) == 0.0);
           default:
             return false;
         }
@@ -1029,8 +1203,8 @@ bool Value::operator==(const Value &other) const {
           case Bool:
             return other.operator==(*this);
           case Float:
-            return value.ui == (unsigned int)other.value.d &&
-                   ((other.value.d - (int)other.value.d) == 0.0);
+            return value.ui == (uint64_t)other.value.d &&
+                   ((other.value.d - (uint64_t)other.value.d) == 0.0);
           default:
             return false;
         }
@@ -1051,6 +1225,202 @@ bool Value::operator==(const Value &other) const {
   return false;
 }
 
+bool Value::operator<(const Value &other) const {
+  if (type == other.type) {
+    switch (type) {
+      case Undefined:
+        return false;
+      case shcore::Null:
+        return false;
+      case Bool:
+        return value.b < other.value.b;
+      case Integer:
+        return value.i < other.value.i;
+      case UInteger:
+        return value.ui < other.value.ui;
+      case Float:
+        return value.d < other.value.d;
+      case String:
+        return *value.s < *other.value.s;
+      case Object:
+        return **value.o < **other.value.o;
+      case Array:
+        return **value.array < **other.value.array;
+      case Map:
+        return **value.map < **other.value.map;
+      case MapRef:
+        return *value.mapref->lock() < *other.value.mapref->lock();
+      case Function:
+        // NOTE: not implemented, it's not possible to order functions
+        return false;
+    }
+  } else {
+    // with type conversion
+    switch (type) {
+      case Undefined:
+        return false;
+      case shcore::Null:
+        return false;
+      case Bool:
+        switch (other.type) {
+          case Integer:
+            if (other.value.i == 1)
+              return value.b < true;
+            else if (other.value.i == 0)
+              return false;
+            return false;
+          case UInteger:
+            if (other.value.ui == 1)
+              return value.b < true;
+            else if (other.value.ui == 0)
+              return false;
+            return false;
+          case Float:
+            if (other.value.d == 1.0)
+              return value.b < true;
+            else if (other.value.d == 0.0)
+              return false;
+            return false;
+          default:
+            return false;
+        }
+      case Integer:
+        switch (other.type) {
+          case Bool:
+            return other.operator<(*this);
+          case Float:
+            return value.i < (int64_t)other.value.d ||
+                   (value.i < (int64_t)other.value.d &&
+                    ((other.value.d - (int64_t)other.value.d) > 0.0));
+          default:
+            return false;
+        }
+      case UInteger:
+        switch (other.type) {
+          case Bool:
+            return other.operator<(*this);
+          case Float:
+            return value.ui < (uint64_t)other.value.d ||
+                   (value.ui == (uint64_t)other.value.d &&
+                    ((other.value.d - (uint64_t)other.value.d) > 0.0));
+          default:
+            return false;
+        }
+      case Float:
+        switch (other.type) {
+          case Bool:
+            return other.operator<(*this);
+          case Integer:
+          case UInteger:
+            return other.operator<(*this);
+          default:
+            return false;
+        }
+      default:
+        return false;
+    }
+  }
+  return false;
+}
+
+bool Value::operator<=(const Value &other) const {
+  if (type == other.type) {
+    switch (type) {
+      case Undefined:
+        return true;  // undefined == undefined is true
+      case shcore::Null:
+        return true;
+      case Bool:
+        return value.b <= other.value.b;
+      case Integer:
+        return value.i <= other.value.i;
+      case UInteger:
+        return value.ui <= other.value.ui;
+      case Float:
+        return value.d <= other.value.d;
+      case String:
+        return *value.s <= *other.value.s;
+      case Object:
+        return **value.o <= **other.value.o;
+      case Array:
+        return **value.array <= **other.value.array;
+      case Map:
+        return **value.map <= **other.value.map;
+      case MapRef:
+        return *value.mapref->lock() <= *other.value.mapref->lock();
+      case Function:
+        // NOTE: not implemented, it's not possible to order functions
+        return **value.func == **other.value.func;
+    }
+  } else {
+    // with type conversion
+    switch (type) {
+      case Undefined:
+        return false;
+      case shcore::Null:
+        return false;
+      case Bool:
+        switch (other.type) {
+          case Integer:
+            if (other.value.i == 1)
+              return true;
+            else if (other.value.i == 0)
+              return value.b <= false;
+            return false;
+          case UInteger:
+            if (other.value.ui == 1)
+              return true;
+            else if (other.value.ui == 0)
+              return value.b <= false;
+            return false;
+          case Float:
+            if (other.value.d == 1.0)
+              return true;
+            else if (other.value.d == 0.0)
+              return value.b <= false;
+            return false;
+          default:
+            return false;
+        }
+      case Integer:
+        switch (other.type) {
+          case Bool:
+            return other.operator<=(*this);
+          case Float:
+            return value.i < (int64_t)other.value.d ||
+                   (value.i == (int64_t)other.value.d &&
+                    ((other.value.d - (int64_t)other.value.d) >= 0.0));
+          default:
+            return false;
+        }
+      case UInteger:
+        switch (other.type) {
+          case Bool:
+            return other.operator<=(*this);
+          case Float:
+            return value.ui < (uint64_t)other.value.d ||
+                   (value.ui == (uint64_t)other.value.d &&
+                    ((other.value.d - (uint64_t)other.value.d) >= 0.0));
+          default:
+            return false;
+        }
+      case Float:
+        switch (other.type) {
+          case Bool:
+            return other.operator<=(*this);
+          case Integer:
+          case UInteger:
+            return other.operator<=(*this);
+          default:
+            return false;
+        }
+      default:
+        return false;
+    }
+  }
+  return false;
+}
+
 std::string Value::json(bool pprint) const {
   std::stringstream s;
   JSON_dumper dumper(pprint);
@@ -1094,14 +1464,9 @@ std::string &Value::append_descr(std::string &s_out, int indent,
     case UInteger:
       s_out += std::to_string(value.ui);
       break;
-    case Float: {
-      char buffer[32];
-      size_t len;
-      len = my_gcvt(value.d, MY_GCVT_ARG_DOUBLE, sizeof(buffer) - 1, buffer,
-                    NULL);
-      s_out.append(buffer, len);
+    case Float:
+      s_out += shcore::dtoa(value.d);
       break;
-    }
     case String:
       if (quote_strings) {
         s_out += quote_string(*value.s, quote_strings);
@@ -1161,8 +1526,7 @@ std::string &Value::append_descr(std::string &s_out, int indent,
       s_out.append("mapref");
       break;
     case Function:
-      // TODO:
-      // value.func->get()->append_descr(s_out, pprint);
+      value.func->get()->append_descr(&s_out, indent, quote_strings);
       break;
   }
   return s_out;
@@ -1275,8 +1639,7 @@ std::string &Value::append_repr(std::string &s_out) const {
     case MapRef:
       break;
     case Function:
-      // TODO:
-      // value.func->get()->append_repr(s_out);
+      value.func->get()->append_repr(&s_out);
       break;
   }
   return s_out;
@@ -1460,7 +1823,7 @@ std::string Value::as_string() const {
 
 std::string Value::yaml() const { return "---\n" + yaml(0) + "\n"; }
 
-std::string Value::yaml(int indent) const {
+std::string Value::yaml(int init_indent) const {
   // implementation based on: http://yaml.org/spec/1.2/spec.html
   static constexpr size_t k_indent_size = 2;
 
@@ -1620,10 +1983,10 @@ std::string Value::yaml(int indent) const {
     case Value_type::Object:
     case Value_type::Function:
       // we treat these as scalars
-      return string2yaml(descr(), indent);
+      return string2yaml(descr(), init_indent);
 
     case Value_type::String:
-      return string2yaml(*value.s, indent);
+      return string2yaml(*value.s, init_indent);
 
     case Value_type::Array: {
       std::string array;
@@ -1633,11 +1996,11 @@ std::string Value::yaml(int indent) const {
         if (first_item) {
           first_item = false;
         } else {
-          new_line(&array, indent);
+          new_line(&array, init_indent);
         }
 
         array += "-";
-        const auto sub_yaml = v.yaml(indent + 1);
+        const auto sub_yaml = v.yaml(init_indent + 1);
 
         if (!sub_yaml.empty()) {
           array += " " + sub_yaml;
@@ -1648,10 +2011,10 @@ std::string Value::yaml(int indent) const {
     }
 
     case Value_type::Map:
-      return map2yaml(*value.map, indent);
+      return map2yaml(*value.map, init_indent);
 
     case Value_type::MapRef:
-      return map2yaml(value.mapref->lock(), indent);
+      return map2yaml(value.mapref->lock(), init_indent);
   }
 
   throw std::logic_error("Type '" + type_name(type) + "' was not handled.");
@@ -1929,10 +2292,17 @@ bool Argument_map::validate_keys(const std::set<std::string> &mandatory_keys,
   return missing_keys.empty() && invalid_keys.empty();
 }
 
-Option_unpacker::Option_unpacker(const Dictionary_t &options)
-    : m_options(options) {
-  if (m_options)
+Option_unpacker::Option_unpacker(const Dictionary_t &options) {
+  set_options(options);
+}
+
+void Option_unpacker::set_options(const Dictionary_t &options) {
+  m_options = options;
+  m_unknown.clear();
+  m_missing.clear();
+  if (m_options) {
     for (const auto &opt : *m_options) m_unknown.insert(opt.first);
+  }
 }
 
 Value Option_unpacker::get_required(const char *name, Value_type type) {
@@ -2050,4 +2420,24 @@ void Object_bridge::append_json(JSON_dumper &dumper) const {
   dumper.end_object();
 }
 
+std::string &Function_base::append_descr(std::string *s_out, int /* indent */,
+                                         int /* quote_strings */) const {
+  const auto &n = name();
+  s_out->append("<Function").append(n.empty() ? "" : ":" + n).append(">");
+  return *s_out;
+}
+
+std::string &Function_base::append_repr(std::string *s_out) const {
+  return append_descr(s_out);
+}
+
+void Function_base::append_json(JSON_dumper *dumper) const {
+  std::string repr;
+  dumper->append_string(append_repr(&repr));
+}
+
+std::ostream &operator<<(std::ostream &os, const Value &v) {
+  return os << v.descr() << " (" << type_name(v.type) << ")";
+}
+
 }  // namespace shcore
