@@ -45,7 +45,7 @@ extern "C" const char *g_test_home;
 namespace mysqlsh {
 namespace dump {
 
-using ::testing::ContainsRegex;
+using ::testing::AnyOf;
 using ::testing::HasSubstr;
 using ::testing::Not;
 
@@ -705,12 +705,10 @@ TEST_F(Schema_dumper_test, dump_grants) {
 
   EXPECT_THAT(out, HasSubstr("'second'"));
 
-  auto filtered = Schema_dumper::preprocess_users_script(
-      out,
-      [](const std::string &user) {
+  auto filtered =
+      Schema_dumper::preprocess_users_script(out, [](const std::string &user) {
         return !shcore::str_beginswith(user, "'second'");
-      },
-      {});
+      });
   EXPECT_THAT(filtered, Not(HasSubstr("'second'")));
 
   // cleanup
@@ -819,14 +817,16 @@ TEST_F(Schema_dumper_test, dump_filtered_grants) {
   }
 
   if (_target_server_version >= mysqlshdk::utils::Version(8, 0, 20)) {
+    const std::string da_dumper =
+        "CREATE ROLE IF NOT EXISTS 'da_dumper'@'%';\n"
+        "ALTER USER 'da_dumper'@'%' IDENTIFIED WITH "
+        "'caching_sha2_password' REQUIRE NONE PASSWORD EXPIRE ACCOUNT "
+        "LOCK PASSWORD HISTORY DEFAULT PASSWORD REUSE INTERVAL DEFAULT "
+        "PASSWORD REQUIRE CURRENT DEFAULT;";
     EXPECT_THAT(
-        out,
-        ContainsRegex(
-            "CREATE ROLE IF NOT EXISTS ['`]da_dumper['`]@['`]%['`];\n"
-            "ALTER USER ['`]da_dumper['`]@['`]%['`] IDENTIFIED WITH "
-            "'caching_sha2_password' REQUIRE NONE PASSWORD EXPIRE ACCOUNT "
-            "LOCK PASSWORD HISTORY DEFAULT PASSWORD REUSE INTERVAL DEFAULT "
-            "PASSWORD REQUIRE CURRENT DEFAULT;"));
+        out, AnyOf(HasSubstr(da_dumper),
+                   HasSubstr(shcore::str_replace(da_dumper, "'da_dumper'@'%'",
+                                                 "`da_dumper`@`%`"))));
 
     EXPECT_THAT(
         out,
@@ -843,10 +843,11 @@ TEST_F(Schema_dumper_test, dump_filtered_grants) {
             "CREATE USER, EVENT, TRIGGER ON `mysql`.* FROM `dave`@`%`;"));
   }
 
-  EXPECT_THAT(
-      out,
-      ContainsRegex(
-          "CREATE USER IF NOT EXISTS ['`]dumptestuser['`]@['`]localhost['`]"));
+  const std::string dumptestuser =
+      "CREATE USER IF NOT EXISTS 'dumptestuser'@'localhost'";
+  EXPECT_THAT(out,
+              AnyOf(HasSubstr(dumptestuser),
+                    HasSubstr(shcore::str_replace(dumptestuser, "'", "`"))));
   EXPECT_THAT(out, HasSubstr("-- begin user 'abr@dab'@'localhost'"));
 
   if (_target_server_version >= mysqlshdk::utils::Version(8, 0, 20)) {
@@ -1647,5 +1648,89 @@ TEST_F(Schema_dumper_test, check_object_for_definer) {
   }
 }
 
+TEST_F(Schema_dumper_test, include_grant) {
+  // BUG#33406711
+  // setup
+  Instance_cache cache;
+
+  cache.schemas["s"].tables["t"].comment = "";
+  cache.schemas["s"].functions.emplace("f");
+  cache.schemas["s"].procedures.emplace("p");
+
+  Schema_dumper sd(session);
+  sd.use_cache(&cache);
+
+  // only GRANT/REVOKE statements are supported
+  EXPECT_THROW_LIKE(sd.include_grant("CREATE USER"), std::runtime_error,
+                    "Expected GRANT or REVOKE statement");
+
+  // GRANT/REVOKE role statements are always included
+  EXPECT_TRUE(sd.include_grant("GRANT role to `user`@`host`"));
+  EXPECT_TRUE(sd.include_grant("REVOKE role FROM 'user'@'host'"));
+
+  // GRANT/REVOKE PROXY statements are always included
+  EXPECT_TRUE(sd.include_grant("grant PROXY ON admin@host TO 'user'@host"));
+  EXPECT_TRUE(sd.include_grant("REVOKE PROxY on admin@host FROM user@host"));
+
+  // global GRANT/REVOKE statements are always included
+  EXPECT_TRUE(sd.include_grant("GRANT SELECT ON *.*"));
+  EXPECT_TRUE(sd.include_grant("GRANT ALTER ON *.*"));
+  EXPECT_TRUE(sd.include_grant("GRANT ALTER ROUTINE ON *.*"));
+  EXPECT_TRUE(sd.include_grant("GRANT EXECUTE ON *.*"));
+
+  // global statements on system schemas are always included
+  for (const std::string schema : {
+           "information_schema",
+           "mysql",
+           "ndbinfo",
+           "performance_schema",
+           "sys",
+       }) {
+    EXPECT_TRUE(sd.include_grant("GRANT ALTER ON " + schema + ".*"));
+    EXPECT_TRUE(sd.include_grant("GRANT ALTER ROUTINE ON " + schema + ".*"));
+    EXPECT_TRUE(sd.include_grant("GRANT EXECUTE ON " + schema + ".*"));
+    EXPECT_TRUE(sd.include_grant("REVOKE SELECT ON " + schema + ".table"));
+    EXPECT_TRUE(sd.include_grant("GRANT SELECT ON TABLE " + schema + ".table"));
+    EXPECT_TRUE(sd.include_grant("REVOKE EXECUTE ON " + schema + ".function"));
+    EXPECT_TRUE(
+        sd.include_grant("GRANT ALTER ROUTINE ON " + schema + ".function"));
+    EXPECT_TRUE(
+        sd.include_grant("REVOKE EXECUTE ON FUNCTION " + schema + ".function"));
+    EXPECT_TRUE(sd.include_grant("GRANT ALTER ROUTINE ON PROCEDURE " + schema +
+                                 ".procedure"));
+  }
+
+  // statements on schema which is not in the cache are excluded
+  EXPECT_FALSE(sd.include_grant("GRANT ALTER ON x.*"));
+  EXPECT_FALSE(sd.include_grant("GRANT ALTER ROUTINE ON x.*"));
+  EXPECT_FALSE(sd.include_grant("GRANT EXECUTE ON x.*"));
+  EXPECT_FALSE(sd.include_grant("REVOKE SELECT ON x.table"));
+  EXPECT_FALSE(sd.include_grant("GRANT SELECT ON TABLE x.table"));
+  EXPECT_FALSE(sd.include_grant("REVOKE EXECUTE ON x.function"));
+  EXPECT_FALSE(sd.include_grant("GRANT ALTER ROUTINE ON x.function"));
+  EXPECT_FALSE(sd.include_grant("REVOKE EXECUTE ON FUNCTION x.function"));
+  EXPECT_FALSE(
+      sd.include_grant("GRANT ALTER ROUTINE ON PROCEDURE x.procedure"));
+
+  // statements on objects which are in the cache are included
+  EXPECT_TRUE(sd.include_grant("GRANT ALTER ON s.*"));
+  EXPECT_TRUE(sd.include_grant("GRANT ALTER ROUTINE ON s.*"));
+  EXPECT_TRUE(sd.include_grant("GRANT EXECUTE ON s.*"));
+  EXPECT_TRUE(sd.include_grant("REVOKE SELECT ON s.t"));
+  EXPECT_TRUE(sd.include_grant("GRANT SELECT ON TABLE s.t"));
+  EXPECT_TRUE(sd.include_grant("REVOKE EXECUTE ON s.f"));
+  EXPECT_TRUE(sd.include_grant("GRANT ALTER ROUTINE ON s.f"));
+  EXPECT_TRUE(sd.include_grant("REVOKE EXECUTE ON FUNCTION s.f"));
+  EXPECT_TRUE(sd.include_grant("GRANT ALTER ROUTINE ON PROCEDURE s.p"));
+
+  // statements on objects which are not in the cache are excluded
+  EXPECT_FALSE(sd.include_grant("REVOKE SELECT ON s.f"));
+  EXPECT_FALSE(sd.include_grant("GRANT SELECT ON TABLE s.p"));
+  EXPECT_FALSE(sd.include_grant("REVOKE EXECUTE ON s.t"));
+  EXPECT_FALSE(sd.include_grant("GRANT ALTER ROUTINE ON s.t"));
+  EXPECT_FALSE(sd.include_grant("REVOKE EXECUTE ON FUNCTION s.t"));
+  EXPECT_FALSE(sd.include_grant("GRANT ALTER ROUTINE ON PROCEDURE s.t"));
+}
+
 }  // namespace dump
 }  // namespace mysqlsh
