@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2018, 2020, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -21,14 +21,16 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <Windows.h>
 #endif
 
+#include "my_config.h"
 #include "mysqlshdk/libs/config/config_file.h"
 #include "mysqlshdk/libs/utils/nullable.h"
 #include "mysqlshdk/libs/utils/utils_file.h"
 #include "mysqlshdk/libs/utils/utils_path.h"
+#include "mysqlshdk/libs/utils/utils_string.h"
 #include "unittest/test_utils.h"
 #include "unittest/test_utils/mocks/gmock_clean.h"
 
@@ -267,7 +269,7 @@ TEST_F(ConfigFileTest, test_read) {
     cfg_path = shcore::path::join_path(m_option_files_basedir,
                                        "my_include_error3.cnf");
     EXPECT_THROW_LIKE(cfg.read(cfg_path), std::runtime_error,
-                      "Unable to get absolute path for '");
+                      "Cannot open file: ");
   }
   {
     // Test error including file with an invalid format (parsing error).
@@ -408,14 +410,16 @@ TEST_F(ConfigFileTest, test_write) {
     cfg.read(cfg_path);
     EXPECT_THROW(cfg.write(no_write_perm_cfg_path), std::runtime_error);
 
-#ifdef WIN32
+#ifdef _WIN32
     // NOTE: On Windows, read-only attribute (previously set) must be removed
     // before deleting the file, otherwise an Access is denied error is
     // issued.
-    auto dwAttrs = GetFileAttributes(no_write_perm_cfg_path.c_str());
+    const auto wide_no_write_perm_cfg_path =
+        shcore::utf8_to_wide(no_write_perm_cfg_path);
+    auto attributes = GetFileAttributesW(wide_no_write_perm_cfg_path.c_str());
     // Remove (reset) read-only attribute from the file
-    SetFileAttributes(no_write_perm_cfg_path.c_str(),
-                      dwAttrs & ~FILE_ATTRIBUTE_READONLY);
+    SetFileAttributesW(wide_no_write_perm_cfg_path.c_str(),
+                       attributes & ~FILE_ATTRIBUTE_READONLY);
 #endif
     // Delete test file at the end.
     shcore::delete_file(no_write_perm_cfg_path, true);
@@ -796,4 +800,34 @@ TEST_F(ConfigFileTest, test_constructor) {
   EXPECT_TRUE(cfg_copy.groups() == cfg.groups());
 }
 
+TEST_F(ConfigFileTest, get_default_config_paths) {
+  using mysqlshdk::config::get_default_config_paths;
+  using shcore::OperatingSystem;
+
+  // BUG#30171324 : mysql shell does not know about my.cnf default location
+  // Test default path include /etc/my.cnf and /etc/mysql/my.cnf for all
+  // unix and unix-like systems.
+  auto test_unix_defaults = [](shcore::OperatingSystem os) {
+    SCOPED_TRACE("Test mandatory default paths for '" + to_string(os) + "'");
+    std::vector<std::string> res = get_default_config_paths(os);
+    std::string sysconfdir;
+#ifdef DEFAULT_SYSCONFDIR
+    sysconfdir = std::string(DEFAULT_SYSCONFDIR);
+#endif
+    EXPECT_THAT(res, Contains("/etc/my.cnf"));
+    EXPECT_THAT(res, Contains("/etc/mysql/my.cnf"));
+    if (!sysconfdir.empty()) {
+      EXPECT_THAT(res, Contains(sysconfdir + "/my.cnf"));
+    }
+  };
+
+  std::vector<shcore::OperatingSystem> unix_os_list{
+      shcore::OperatingSystem::DEBIAN, shcore::OperatingSystem::REDHAT,
+      shcore::OperatingSystem::SOLARIS, shcore::OperatingSystem::LINUX,
+      shcore::OperatingSystem::MACOS};
+  for (const auto &unix_os : unix_os_list) {
+    test_unix_defaults(unix_os);
+  }
+}
+
 }  // namespace testing
