@@ -34,6 +34,7 @@ NOTE: Group Replication will communicate with other members using '<<<hostname>>
 <<<(__version_num<80011)?"WARNING: Instance '"+hostname+":"+__mysql_sandbox_port1+"' cannot persist Group Replication configuration since MySQL version "+__version+" does not support the SET PERSIST command (MySQL version >= 8.0.11 required). Please use the dba.configureLocalInstance() command locally to persist the changes.\n":""\>>>
 Creating InnoDB cluster 'test' on '<<<hostname>>>:<<<__mysql_sandbox_port1>>>'...
 
+The MySQL InnoDB cluster is going to be setup in advanced Multi-Primary Mode. Consult its requirements and limitations in https://dev.mysql.com/doc/refman/en/group-replication-limitations.html
 Adding Seed Instance...
 Cluster successfully created. Use Cluster.addInstance() to add MySQL instances.
 At least 3 instances are needed for the cluster to be able to withstand up to
@@ -434,6 +435,10 @@ one server failure.
             {
                 "option": "disableClone",
                 "value": <<<(__version_num>=80017)?"false":"true">>>
+            },
+            {
+                "option": "replicationAllowedHost", 
+                "value": "%"
             }
         ],
         "tags": {
@@ -557,6 +562,10 @@ one server failure.
             {
                 "option": "disableClone",
                 "value": true
+            },
+            {
+                "option": "replicationAllowedHost", 
+                "value": "%"
             }
         ],
         "tags": {
@@ -676,6 +685,10 @@ one server failure.
             {
                 "option": "disableClone",
                 "value": true
+            },
+            {
+                "option": "replicationAllowedHost", 
+                "value": "%"
             }
         ],
         "tags": {
@@ -799,9 +812,3 @@ one server failure.
 
 //@ IPv6 on ipWhitelist is not supported below 8.0.14 WL#12758 {VER(< 8.0.14)}
 ||Invalid value for ipWhitelist '::1': IPv6 not supported (version >= 8.0.14 required for IPv6 support). (ArgumentError)
-
-//@ IPv6 on groupSeeds is not supported below 8.0.14 - 1 WL#12758 {VER(< 8.0.14)}
-||Instance does not support the following groupSeed value :'[::1]:<<<__mysql_sandbox_gr_port2>>>'. IPv6 addresses/hostnames are only supported by Group Replication from MySQL version >= 8.0.14 and the target instance version is <<<__version>>>. (ArgumentError)
-
-//@ IPv6 on groupSeeds is not supported below 8.0.14 - 2 WL#12758 {VER(< 8.0.14)}
-||Instance does not support the following groupSeed values :'[::1]:<<<__mysql_sandbox_gr_port2>>>, [fe80::7e36:f49a:63c8:8ad6]:<<<__mysql_sandbox_gr_port1>>>'. IPv6 addresses/hostnames are only supported by Group Replication from MySQL version >= 8.0.14 and the target instance version is <<<__version>>>. (ArgumentError)
