@@ -12,8 +12,9 @@ use the following wildcards:
 
 The following are the main help categories:
 
- - AdminAPI       Introduces to the dba global object and the InnoDB cluster
-                  administration API.
+ - AdminAPI       The AdminAPI is an API that enables configuring and managing
+                  InnoDB Clusters, ReplicaSets, ClusterSets, among other
+                  things.
  - Shell Commands Provides details about the available built-in shell commands.
  - ShellAPI       Contains information about the shell and util global objects
                   as well as the mysql module that enables executing SQL on
@@ -44,6 +45,8 @@ The following shell commands are available:
  - \                   Start multi-line input when in SQL mode.
  - \connect    (\c)    Connects the shell to a MySQL server and assigns the
                        global session.
+ - \disconnect         Disconnects the global session.
+ - \edit       (\e)    Launch a system editor to edit a command to be executed.
  - \exit               Exits the MySQL Shell, same as \quit.
  - \help       (\?,\h) Prints help information about a specific topic.
  - \history            View and edit command line history.
@@ -62,6 +65,7 @@ The following shell commands are available:
  - \sql                Executes SQL statement or switches to SQL processing
                        mode when no statement is given.
  - \status     (\s)    Print information about the current global session.
+ - \system     (\!)    Execute a system shell command.
  - \use        (\u)    Sets the active schema.
  - \warnings   (\W)    Show warnings after every statement.
  - \watch              Executes the given report with provided options and
@@ -71,7 +75,7 @@ GLOBAL OBJECTS
 
 The following modules and objects are ready for use when the shell starts:
 
- - dba      Used for InnoDB cluster administration.
+ - dba      Used for InnoDB Cluster, ReplicaSet, and ClusterSet administration.
  - mysql    Support for connecting to MySQL servers using the classic MySQL
             protocol.
  - mysqlx   Used to work with X Protocol sessions using the MySQL X DevAPI.
@@ -109,8 +113,9 @@ use the following wildcards:
 
 The following are the main help categories:
 
- - AdminAPI       Introduces to the dba global object and the InnoDB cluster
-                  administration API.
+ - AdminAPI       The AdminAPI is an API that enables configuring and managing
+                  InnoDB Clusters, ReplicaSets, ClusterSets, among other
+                  things.
  - Shell Commands Provides details about the available built-in shell commands.
  - ShellAPI       Contains information about the shell and util global objects
                   as well as the mysql module that enables executing SQL on
@@ -156,6 +161,8 @@ The following shell commands are available:
  - \                   Start multi-line input when in SQL mode.
  - \connect    (\c)    Connects the shell to a MySQL server and assigns the
                        global session.
+ - \disconnect         Disconnects the global session.
+ - \edit       (\e)    Launch a system editor to edit a command to be executed.
  - \exit               Exits the MySQL Shell, same as \quit.
  - \G                  Send command to mysql server, display result vertically.
  - \g                  Send command to mysql server.
@@ -176,6 +183,7 @@ The following shell commands are available:
  - \sql                Executes SQL statement or switches to SQL processing
                        mode when no statement is given.
  - \status     (\s)    Print information about the current global session.
+ - \system     (\!)    Execute a system shell command.
  - \use        (\u)    Sets the active schema.
  - \warnings   (\W)    Show warnings after every statement.
  - \watch              Executes the given report with provided options and
@@ -209,26 +217,93 @@ Use \? \help for additional details.
 Switching to Python mode...
 
 #@<OUT> Help on Admin API Category
-MySQL InnoDB cluster provides a complete high availability solution for MySQL.
-
-The AdminAPI is an interactive API that enables configuring and administering
-InnoDB clusters.
-
-Use the dba global object to:
-
-- Verify if a MySQL server is suitable for InnoDB cluster.
-- Configure a MySQL server to be used as an InnoDB cluster instance.
-- Create an InnoDB cluster.
-- Get a handle for performing operations on an InnoDB cluster.
-- Other InnoDB cluster maintenance tasks.
-
-In the AdminAPI, an InnoDB cluster is represented as an instance of the Cluster
-class.
+The AdminAPI can be used interactively from the MySQL Shell prompt and
+non-interactively from JavaScript and Python scripts and directly from the
+command line.
 
 For more information about the dba object use: \? dba
 
+In the AdminAPI, an InnoDB Cluster is represented as an instance of the Cluster
+class, while ReplicaSets are represented as an instance of the ReplicaSet
+class, and ClusterSets are represented as an instance of the ClusterSet class.
+
 For more information about the Cluster class use: \? Cluster
 
+For more information about the ClusterSet class use: \? ClusterSet
+
+For more information about the ReplicaSet class use: \? ReplicaSet
+
+Scripting
+
+Through the JavaScript and Python bindings of the MySQL Shell, the AdminAPI can
+be used from scripts, which can in turn be used interactively or
+non-interactively. To execute a script, use the -f command line option,
+followed by the script file path. Options that follow the path are passed
+directly to the script being executed, which can access them from sys.argv
+    mysqlsh root@localhost -f myscript.py arg1 arg2
+
+If the script finishes successfully, the Shell will exit with code 0, while
+uncaught exceptions/errors cause it to exist with a non-0 code.
+
+By default, the AdminAPI enables interactivity, which will cause functions to
+prompt for missing passwords, confirmations and bits of information that cannot
+be obtained automatically.
+
+Prompts can be completely disabled with the --no-wizard command line option or
+using the "interactive" boolean option available in some of functions. If
+interactivity is disabled and some information is missing (e.g. a password), an
+error will be raised instead of being prompted.
+
+Secure Password Handling
+
+Passwords can be safely stored locally, using the system's native secrets
+storage functionality (or login-paths in Linux). Whenever the Shell needs a
+password, it will first query for the password in the system, before prompting
+for it.
+
+Passwords can be stored during interactive use, by confirming in the Store
+Password prompt. They can also be stored programmatically, using the
+shell.store_credential() function.
+
+You can also use environment variables to pass information to your scripts. In
+JavaScript, the os.getenv() function can be used to access them.
+
+Command Line Interface
+
+In addition to the scripting interface, the MySQL Shell supports generic
+command line integration, which allows calling AdminAPI functions directly from
+the system shell (e.g. bash). Examples:
+    $ mysqlsh -- dba configure-instance root@localhost
+
+    is equivalent to:
+
+    > dba.configureInstance("root@localhost")
+
+    $ mysqlsh root@localhost -- cluster status --extended
+
+    is equivalent to:
+
+    > dba.getCluster().status({extended:true})
+
+The mapping from AdminAPI function signatures works as follows:
+
+- The first argument after a -- can be a shell global object, such as dba. As a
+  special case, cluster and rs are also accepted.
+- The second argument is the name of the function of the object to be called.
+  The naming convention is automatically converted from camelCase/snake_case to
+  lower case separated by dashes.
+- The rest of the arguments are used in the same order as their JS/Python
+  counterparts. Instances can be given as URIs. Option dictionaries can be
+  passed as --options, where the option name is the same as in JS/Python.
+
+OBJECTS
+ - dba InnoDB Cluster, ReplicaSet, and ClusterSet management functions.
+
+CLASSES
+ - Cluster    Represents an InnoDB Cluster.
+ - ClusterSet Represents an InnoDB ClusterSet.
+ - ReplicaSet Represents an InnoDB ReplicaSet.
+
 #@<OUT> Help on shell commands
 The shell commands allow executing specific operations including updating the
 shell configuration.
@@ -238,6 +313,8 @@ The following shell commands are available:
  - \                   Start multi-line input when in SQL mode.
  - \connect    (\c)    Connects the shell to a MySQL server and assigns the
                        global session.
+ - \disconnect         Disconnects the global session.
+ - \edit       (\e)    Launch a system editor to edit a command to be executed.
  - \exit               Exits the MySQL Shell, same as \quit.
  - \help       (\?,\h) Prints help information about a specific topic.
  - \history            View and edit command line history.
@@ -256,6 +333,7 @@ The following shell commands are available:
  - \sql                Executes SQL statement or switches to SQL processing
                        mode when no statement is given.
  - \status     (\s)    Print information about the current global session.
+ - \system     (\!)    Execute a system shell command.
  - \use        (\u)    Sets the active schema.
  - \warnings   (\W)    Show warnings after every statement.
  - \watch              Executes the given report with provided options and
@@ -312,6 +390,10 @@ No help items found matching 'unknown'
 #@<OUT> Help on topic with several matches
 Found several entries matching session
 
+The following topics were found at the AdminAPI category:
+
+- dba.session
+
 The following topics were found at the X DevAPI category:
 
 - mysqlx.Collection.session
@@ -322,7 +404,7 @@ The following topics were found at the X DevAPI category:
 
 For help on a specific topic use: \? <topic>
 
-e.g.: \? mysqlx.Collection.session
+e.g.: \? dba.session
 
 #@<OUT> Help for sandbox operations
 Found several entries matching *sandbox*
@@ -387,12 +469,12 @@ command line, by following a specific syntax.
 
 SYNTAX:
 
-  mysqlsh [options] -- <object> <method> [arguments]
+  mysqlsh [options] -- <object> <operation> [arguments]
 
 WHERE:
 
 - object - a string identifying shell object to be called.
-- method - a string identifying method to be called on the object.
+- operation - a string identifying method to be called on the object.
 - arguments - arguments passed to the called method.
 
 DETAILS:
@@ -405,7 +487,7 @@ The following objects can be called using this format:
 - shell.options - shell.options object.
 - util - util global object
 
-The method name can be given in the following naming styles:
+The operation name can be given in the following naming styles:
 
 - Camel case: (e.g. createCluster, checkForServerUpgrade)
 - Kebab case: (e.g. create-cluster, check-for-server-upgrade)
@@ -445,18 +527,6 @@ specified either using camelCaseNaming or kebab-case-naming, examples:
 If the value is not provided the option name will be handled as a boolean
 option with a default value of TRUE.
 
-Grouping Named Arguments
-
-It is possible to create a group of named arguments by enclosing them between
-curly braces. This group would be handled as a positional argument at the
-position where the opening curly brace was found, it will be passed to the
-function as a dictionary containing all the named arguments defined on the
-group.
-
-Named arguments that are not placed inside curly braces (independently of the
-position on the command line), are packed a single dictionary, passed as a last
-parameter on the method call.
-
 Following are some examples of command line calls as well as how they are
 mapped to the API method call.
 
@@ -477,7 +547,7 @@ $ mysqlsh -- util checkForServerUpgrade root@localhost --outputFormat=JSON
       mysql-js> util.checkForServerUpgrade("root@localhost",{outputFormat:
       "JSON"})
 
-$ mysqlsh -- util check-for-server-upgrade { --user=root --host=localhost }
+$ mysqlsh -- util check-for-server-upgrade --user=root --host=localhost
 --password=
       mysql-js> util.checkForServerUpgrade({user:"root", host:"localhost"},
       {password:""})
